/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package examples.invoice.persistclass;

import org.objectweb.jorm.facility.naming.basidir.BasidBinder;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PClassMappingCtrl;
import org.objectweb.jorm.api.PMapCluster;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

import java.util.HashMap;
import java.util.Iterator;

import examples.invoice.applications.JormInit;

/**
 * @author	P. Dechamboux
 */
public class InvoiceHelper implements MapHelper {
	public final static String CN = "examples.invoice.persistclass.Invoice";
	private LoggerFactory loggerFactory;
	private HashMap pobjMap = new HashMap();
	private AddressHelper addressHelper;
	private ProductHelper productHelper;
	private ProdUnitsHelper prodUnitsHelper;

	public InvoiceHelper(AddressHelper ah, ProductHelper ph, ProdUnitsHelper puh) {
		addressHelper = ah;
		productHelper = ph;
		prodUnitsHelper = puh;
	}

	/**
	 * Prepares the mapping of the Address persistent class for the
	 * AddressAppli or other applications.
	 */
	public void map(PMapper m, LoggerFactory lf, byte clact)
			throws PException {
		addressHelper.map(m, lf, clact);
		productHelper.map(m, lf, clact);
		prodUnitsHelper.map(m, lf, clact);
		loggerFactory = lf;
		Logger logger = lf.getLogger(CN);
		PClassMapping pcm = m.lookup(CN);
		// Perform mapping only if it has not been already done
		if (pcm == null) {
			// puset Set mapping definition
			PBinder sbinder = new BasidBinder(PNamingContext.CTINT);
			PClassMapping sm;
			try {
                String rootmn;
                int end = m.getMapperName().indexOf(".");
                if (end == -1) {
                    rootmn = m.getMapperName();
                } else {
                    rootmn = m.getMapperName().substring(0, end);
                }
				sm = m.createGenClassMapping();
			} catch (Exception e) {
				e.printStackTrace();
				throw new PException("Cannot create PClassMapping (probably a ClassLoader problem).");
			}
			((Loggable) sm).setLoggerFactory(loggerFactory);
			sm.setPBinder(sbinder);
			sbinder.setPClassMapping(sm);
            sbinder.setCacheManager(JormInit.getInstance().getCacheManager());
			((PClassMappingCtrl) sm).setPNameCoder(m.lookup(prodUnitsHelper.CN).getPBinder());
			// Invoice mapping definition
			PBinder binder = new BasidBinder(PNamingContext.CTINT);
			try {
				pcm = (PClassMapping) Class.forName(m.cn2mn(CN) +
													PMapper.PCLASSMAPPINGAPPENDER)
						.newInstance();
			} catch (Exception e) {
				throw new PException("Cannot create PClassMapping (probably a ClassLoader problem).");
			}
			pcm.setPBinder(binder);
			binder.setPClassMapping(pcm);
            binder.setCacheManager(JormInit.getInstance().getCacheManager());
			((PClassMappingCtrl) pcm).setPNameCoder("address", m.lookup(addressHelper.CN).getPBinder());
			((PClassMappingCtrl) pcm).setGenClassMapping("puset", sm);
			((PClassMappingCtrl) pcm).setPNameCoder("puset", sm.getPBinder());
			m.map(pcm);
			PMapCluster cl = m.getPMapCluster(CN);
			switch (clact) {
			case PClassMapping.CREATE_STRUCTURE_IF_NEEDED:
				cl.createMappingStructures(false);
				break;
			case PClassMapping.CLEANUP_REMOVEALL:
				cl.deleteMappingStructures();
				cl.createMappingStructures(false);
				break;
			case PClassMapping.CLEANUP_REMOVEDATA:
				cl.deleteData();
				break;
			case PClassMapping.CLEANUP_DONOTHING:
				break;
			}
		}
	}

	public AddressHelper getAddressHelper() {
		if (addressHelper == null)
			System.out.println("WARNING: addressHelper is NULL in invoiceHelper!!");
		return addressHelper;
	}

	public ProdUnitsHelper getProdUnitsHelper() {
		if (prodUnitsHelper == null)
			System.out.println("WARNING: prodUnitsHelper is NULL in invoiceHelper!!");
		return prodUnitsHelper;
	}

	/**
	 * Tries to retrieve an Address knowing its name.
	 */
	public Invoice getInvoice(PMapper m, PName pn) throws PException {
		Invoice res;
		if (pn == null)
			return null;
		PClassMapping pcm = m.lookup(CN);
		PBinding pb = pcm.getPBinder().lookup(pn);
		if (pb == null) {
			Object conn = m.getConnection();
			try {
				pb = pcm.createPBinding();
				pb.bind(pn);
				res = new Invoice(this, conn, pb);
				pobjMap.put(pb, res);
				m.closeConnection(conn);
			} catch (PException pe) {
				m.closeConnection(conn);
				throw pe;
			}
		} else {
			res = (Invoice) pobjMap.get(pb);
		}
		return res;
	}

	/**
	 * Creates a new persistent Invoice.
	 */
	public Invoice createInvoice(PMapper m, int n, String ar, String[] purs)
			throws PException {
		Invoice res;
		PBinding pb = m.lookup(CN).createPBinding();
		Object conn = m.getConnection();
		try {
			PName pn = m.lookup(CN).getPNameCoder("address").decodeString(ar);
			PName[] pus = new PName[purs.length];
			for (int i = 0; i < purs.length; i++)
				pus[i] = m.lookup(CN).getGenClassMapping("puset").getPNameCoder().decodeString(purs[i]);
			res = new Invoice(this, conn, pb, n, pn, pus);
			m.closeConnection(conn);
			pobjMap.put(pb, res);
		} catch (PException pe) {
			m.closeConnection(conn);
			throw pe;
		}
		return res;
	}

	/**
	 * Deletes a persistent Invoice.
	 */
	public boolean deleteInvoice(PMapper m, String id) throws Exception {
		PName pn = m.lookup(CN).getPBinder().decodeString(id);
		Invoice p = getInvoice(m, pn);
		if (p == null)
			return false;
		Object conn = m.getConnection();
		try {
			PBinding pb = p.getPBinding();
			p.delete(conn);
			m.closeConnection(conn);
			pobjMap.remove(pb);
		} catch (PException pe) {
			m.closeConnection(conn);
			throw pe;
		}
		return true;
	}

	/**
	 * Lists all existing Invoices.
	 */
	public Iterator listInvoices(PMapper m) throws Exception {
		Iterator res;
		Object conn = m.getConnection();
		try {
			res = m.lookup(CN).getPNameIterator(conn);
			m.closeConnection(conn);
		} catch (PException pe) {
			m.closeConnection(conn);
			throw pe;
		}
		return res;
	}

	/**
	 * This methods returns the word specified in parameter with the first
	 * character in upper-case. This method is used for the generation of
	 * accessor methods.
	 * @param word	The String which the first character must be set in
	 * 				upper-case
	 * @return		The String whith the first character in upper-case
	 */
	private String upperFL(String word) {
		if (word != null && word.length() > 0)
			return word.substring(0, 1).toUpperCase() + word.substring(1);
		else
			return word;
	}
}
