/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package examples.invoice.persistclass;

import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.facility.naming.basidir.BasidBinder;
import org.objectweb.jorm.facility.naming.polymorphid.PolymorphIdBinder;
import org.objectweb.jorm.facility.naming.polymorphid.PolymorphIdMgrRegistry;
import org.objectweb.jorm.facility.naming.polymorphid.PolymorphIdMgr;
import org.objectweb.jorm.facility.naming.polymorphid.PolymorphIdMgrImpl;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PMapCluster;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

import java.util.HashMap;
import java.util.Iterator;

import examples.invoice.applications.JormInit;

/**
 * @author	P. Dechamboux
 */
public class ProductHelper implements MapHelper {
	public final static String CN = "examples.invoice.persistclass.Product";
	private LoggerFactory loggerFactory;
	private HashMap pobjMap = new HashMap();

	/**
	 * Prepares the mapping of the Address persistent class for the
	 * AddressAppli or other applications.
	 */
	public void map(PMapper m, LoggerFactory lf, byte clact)
		throws PException {
		loggerFactory = lf;
		Logger logger = lf.getLogger(CN);
		// LongGenIncr initialization
		PClassMapping pcm = m.lookup(CN);
		// Perform mapping only if it has not been already done
		if (pcm == null) {
			PolymorphIdMgr pim = PolymorphIdMgrRegistry.getPolymorphIdMgr(m);
			if (pim == null) {
				pim = new PolymorphIdMgrImpl();
				pim.init(m, clact);
				PolymorphIdMgrRegistry.registerPolymorphIdMgr(pim);
			}
			PBinder binder = (PBinder) pim.getPBinder(CN);
			try {
				pcm = (PClassMapping) Class.forName(m.cn2mn(CN) + PMapper.PCLASSMAPPINGAPPENDER).newInstance();
			} catch (Exception e) {
				throw new PException("Cannot create PClassMapping (probably a ClassLoader problem).");
			}
			pcm.setPBinder(binder);
			binder.setPClassMapping(pcm);
            binder.setCacheManager(JormInit.getInstance().getCacheManager());
			m.map(pcm);
		}
		PMapCluster cl = m.getPMapCluster(CN);
		switch (clact) {
		case PClassMapping.CREATE_STRUCTURE_IF_NEEDED:
			cl.createMappingStructures(false);
			break;
		case PClassMapping.CLEANUP_REMOVEALL:
			cl.deleteMappingStructures();
			cl.createMappingStructures(false);
			break;
		case PClassMapping.CLEANUP_REMOVEDATA:
			cl.deleteData();
			break;
		case PClassMapping.CLEANUP_DONOTHING:
			break;
		}
	}

	/**
	 * Tries to retrieve an Address knowing its name.
	 */
	public Product getProduct(PMapper m, PName pn) throws PException {
		Product res;
		if (pn == null)
			return null;
		PClassMapping pcm = m.lookup(CN);
		PBinding pb = pcm.getPBinder().lookup(pn);
		if (pb == null) {
			Object conn = m.getConnection();
			try {
				pb = pcm.createPBinding();
				pb.bind(pn);
				res = new Product(conn, pb);
				pobjMap.put(pb, res);
				m.closeConnection(conn);
			} catch (PException pe) {
				m.closeConnection(conn);
				throw pe;
			}
		} else {
			res = (Product) pobjMap.get(pb);
		}
		return res;
	}

	/**
	 * Creates a new persistent Product.
	 */
	public Product createProduct(PMapper m, String n, char s, String c, float w, float p) throws PException {
		Product res;
		PBinding pb = m.lookup(CN).createPBinding();
		Object conn = m.getConnection();
		try {
			res = new Product(conn, pb, n, s, c, w, p);
			m.closeConnection(conn);
			pobjMap.put(pb, res);
		} catch (PException pe) {
pe.getNestedException().printStackTrace();
			m.closeConnection(conn);
			throw pe;
		}
		return res;
	}

	/**
	 * Deletes a persistent Product.
	 */
	public boolean deleteProduct(PMapper m, String id) throws PException {
		PName pn = m.lookup(CN).getPBinder().decodeString(id);
		Product p = getProduct(m, pn);
		if (p == null)
			return false;
		Object conn = m.getConnection();
		try {
			PBinding pb = p.getPBinding();
			p.delete(conn);
			m.closeConnection(conn);
			pobjMap.remove(pb);
		} catch (PException pe) {
			m.closeConnection(conn);
			throw pe;
		}
		return true;
	}

	/**
	 * Lists all existing Products.
	 */
	public Iterator listProducts(PMapper m) throws PException {
		Iterator res;
		Object conn = m.getConnection();
		try {
			res = m.lookup(CN).getPNameIterator(conn);
			m.closeConnection(conn);
		} catch (PException pe) {
			m.closeConnection(conn);
			throw pe;
		}
		return res;
	}
}
