/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.ClassProject;
import org.objectweb.jorm.metainfo.api.ClassRef;
import org.objectweb.jorm.metainfo.api.CompositeName;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.NameRef;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.metainfo.api.ScalarField;
import org.objectweb.jorm.metainfo.rdb.api.RdbClassMapping;
import org.objectweb.jorm.metainfo.rdb.api.RdbClassMultiMapping;
import org.objectweb.jorm.metainfo.rdb.api.RdbExternalTable;
import org.objectweb.jorm.metainfo.rdb.api.RdbJoin;
import org.objectweb.jorm.metainfo.rdb.api.RdbMapping;
import org.objectweb.jorm.metainfo.rdb.api.RdbTable;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * This example builds the jorm meta information for a simple relation between
 * two persistent object. The A objects has a reference 'myB' to a B object.
 * whereas the B has a reference 'myA' to the A object. This is a one-one
 * relationship.
 * In a mapping point of view the objects are stored on sql table with the
 * following schema
 * table_A(pka1_col, pka2_col, f1_col, fkb1_col, fkb2_col)
 * table_A(pkb1_col, pkb2_col, f2_col)
 *
 * S.Chassande-Barrioz
 */
public class OneOneBiCPK extends JormcRunner {

    public final static String PROJECT_NAME = "jormExamples";
    public final static String A_CLASS_NAME = "org.objectweb.jorm.examples.oobcpk.A";
    public final static String A_PK_COMPOSITE_NAME = "org.objectweb.jorm.examples.oobcpk.APk";
    public final static String B_CLASS_NAME = "org.objectweb.jorm.examples.oobcpk.B";
    public final static String B_PK_COMPOSITE_NAME = "org.objectweb.jorm.examples.oobcpk.BPk";


    public static void main(String[] args) {
        try {
            OneOneBiCPK oobcpk = new OneOneBiCPK(args.length == 0 ? "./" : args[0]);
            logger.log(BasicLevel.INFO, "Example OneOneBiCPK");
            oobcpk.createMIofA();
            oobcpk.createMIofB();
            oobcpk.generate();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR, "", getInner(e));
        }
    }
    public OneOneBiCPK(String output) throws PException {
        super(PROJECT_NAME, output);
    }

    public void createMIofA() throws PException {
        logger.log(BasicLevel.INFO, "Create the meta information of the A class");
        Class aclass = compiler.getMIManager().createClass(A_CLASS_NAME);

        PrimitiveElement ida1 =
                aclass.createPrimitiveElement("ida1", PTypeSpace.STRING);
        PrimitiveElement ida2 =
                aclass.createPrimitiveElement("ida2", PTypeSpace.LONG);
        PrimitiveElement f1 =
                        aclass.createPrimitiveElement("f1", PTypeSpace.STRING);

        // pk composite

        NameDef aNameDef = aclass.createNameDef();
        NameRef nr = aNameDef.createNameRef(getACompositeName());
        nr.addProjection("foo1", ida1.getName());
        nr.addProjection("foo2", ida2.getName());

        // reference to B
        ScalarField idb1 = aclass.createHiddenField("idb1", PTypeSpace.BYTE);
        ScalarField idb2 = aclass.createHiddenField("idb2", PTypeSpace.INT);
        Class bclass = compiler.getMIManager().createClass(B_CLASS_NAME);
        ClassRef bcr = aclass.createClassRef("myB", bclass);
        NameDef myBNameDef = bcr.createRefNameDef();
        NameRef myBnr = myBNameDef.createNameRef(getBCompositeName());
        myBnr.addProjection("bar1", idb1.getName());
        myBnr.addProjection("bar2", idb2.getName());

        // ----------------- Mapping for A -----------------
        ClassProject acp = aclass.createClassProject(PROJECT_NAME);
        RdbMapping am = (RdbMapping) acp.createMapping("rdb");

        RdbClassMapping acm = am.createClassMapping("to-table");
        RdbTable tableA = acm.createRdbTable("table_A");
        //Identifier of the A class
        acm.createIdentifierMapping(aNameDef);

        // Mapping rule of the reference myB
        acm.createReferenceMapping("embedded-target-reference", myBNameDef);

        // Mapping of the ida field over the pka_col
        tableA.createPrimitiveElementMapping(ida1, "pka1_col", null, true);
        tableA.createPrimitiveElementMapping(ida2, "pka2_col", null, true);
        // Mapping of the f1 field over the f1_col
        tableA.createPrimitiveElementMapping(f1, "f1_col", null, true);
        // Mapping of the idb field over the fkb_col
        tableA.createPrimitiveElementMapping(idb1, "fkb1_col", null, false);
        tableA.createPrimitiveElementMapping(idb2, "fkb2_col", null, false);
    }

    public void createMIofB() throws PException {
        logger.log(BasicLevel.INFO, "Create the meta information of the B class");
        Class bclass = compiler.getMIManager().createClass(B_CLASS_NAME);
        PrimitiveElement idb1 =
                bclass.createPrimitiveElement("idb1", PTypeSpace.BYTE);
        PrimitiveElement idb2 =
                bclass.createPrimitiveElement("idb2", PTypeSpace.INT);
        PrimitiveElement f2 =
                bclass.createPrimitiveElement("f2", PTypeSpace.STRING);

        // pk simple
        NameDef bNameDef = bclass.createNameDef();
        NameRef nr = bNameDef.createNameRef(getBCompositeName());
        nr.addProjection("bar1", idb1.getName());
        nr.addProjection("bar2", idb2.getName());

        // reference to A
        ScalarField ida1 = bclass.createHiddenField("ida1", PTypeSpace.STRING);
        ScalarField ida2 = bclass.createHiddenField("ida2", PTypeSpace.LONG);
        Class aclass = compiler.getMIManager().createClass(A_CLASS_NAME);
        ClassRef acr = bclass.createClassRef("myA", aclass);
        NameDef myANameDef = acr.createRefNameDef();
        NameRef myAnr = myANameDef.createNameRef(getACompositeName());
        myAnr.addProjection("foo1", ida1.getName());
        myAnr.addProjection("foo2", ida2.getName());


        // ----------------- Mapping for B -----------------

        ClassProject bcp = bclass.createClassProject(PROJECT_NAME);
        RdbMapping bm = (RdbMapping) bcp.createMapping("rdb");

        RdbClassMultiMapping bcm = bm.createClassMultiMapping("to-table");
        RdbTable tableB = bcm.createRdbTable("table_B");
        bcm.createIdentifierMapping(bNameDef);

        bcm.createReferenceMapping("multi-table", myANameDef);

        tableB.createPrimitiveElementMapping(idb1, "pkb1_col", null, true);
        tableB.createPrimitiveElementMapping(idb2, "pkb2_col", null, true);
        tableB.createPrimitiveElementMapping(f2, "f2_col", null, true);

        //Mapping of the reference to A
        RdbExternalTable extTableA = bcm.createRdbExternalTable("table_A");
        extTableA.setReadOnly(true); // optimisation to avoid writings
        extTableA.createPrimitiveElementMapping(ida1, "pka1_col");
        extTableA.createPrimitiveElementMapping(ida2, "pka2_col");
        RdbJoin joinToA = extTableA.createRdbJoin("myA_Join");
        joinToA.addJoinColumnNames("pkb1_col", "fkb1_col");
        joinToA.addJoinColumnNames("pkb2_col", "fkb2_col");
    }

    public CompositeName getACompositeName() {
        CompositeName acn = compiler.getMIManager()
                .getCompositeName(A_PK_COMPOSITE_NAME);
        if (acn == null) {
            acn = compiler.getMIManager()
                    .createCompositeName(A_PK_COMPOSITE_NAME);
            acn.createCompositeNameField("foo1", PTypeSpace.STRING);
            acn.createCompositeNameField("foo2", PTypeSpace.LONG);
        }
        return acn;
    }
    public CompositeName getBCompositeName() {
        CompositeName bcn = compiler.getMIManager()
                .getCompositeName(B_PK_COMPOSITE_NAME);
        if (bcn == null) {
            bcn = compiler.getMIManager()
                    .createCompositeName(B_PK_COMPOSITE_NAME);
            bcn.createCompositeNameField("bar1", PTypeSpace.BYTE);
            bcn.createCompositeNameField("bar2", PTypeSpace.INT);
        }
        return bcn;
    }

}
