/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.api;

import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;

import java.util.Iterator;

/**
 * The <b>PClassMapping</b> interface defines the behaviour of objects that
 * contain all information required to map a JORM class. It is a PBinder but
 * mainly delegates this behaviour to a nested PBinder that it must reference.
 * It allows the creation or deletion of the persistent structures required to
 * map this class within a particular hosting data store.
 * It also holds references to naming contexts associated to reference fields,
 * as well as to PClassMapping associated to generic class instances of
 * relevant fields.
 * @author	P. Dechamboux
 */
public interface PClassMapping {
	/**
	 * It tells that a cleanup is required for a JORM class with
	 * the removal of existing data as well as hosting structures.
	 */
	final static byte CLEANUP_REMOVEALL = 1;
	/**
	 * It tells that a cleanup is required for a JORM class with
	 * the removal of only existing data.
	 */
	final static byte CLEANUP_REMOVEDATA = 2;
	/**
	 * It permits to specify to create the if it does not already exist. If the
	 * table exist then nothing is done.
	 */
	final static byte CREATE_STRUCTURE_IF_NEEDED = 3;
	/**
	 * It tells that a cleanup is required for a JORM class with
	 * nothing to do on existing data or hosting structures.
	 */
	final static byte CLEANUP_DONOTHING = 4;

	/**
	 * It creates a new PBinding.
	 * @return	The new PBinding with lifecycle state set to
	 * 			LIFECYCLE_NOTBOUND.
	 */
	PBinding createPBinding() throws PException;

	/**
	 * It intialises the class mapping. It especially assigns a mapper as well
	 * as a meta-object describing either a class or a generic class. It also
	 * defines all type information related to this class mapping.
	 * @param mapper	The mapper to be assigned to this class mapping.
	 * @param metaclass	The meta-object describing the associated class. It may
	 * 					be null.
	 */
	void init(PMappingCallback mapper, MetaObject metaclass) throws PException;

	/**
	 * It retrieves the name of the class referenced by this PClassMapping.
	 * should be the same as the one get when calling
	 * "getPBinder().getPType().getJormName()" if the PType exists, that is a
	 * fully qualified name.
	 * @return	The name of this class.
	 */
	String getClassName();

	/**
	 * It retrieves the project name.
	 */
	String getProjectName();

	/**
	 * It gives access to the PClassMapping associated to the element of a
	 * generic class whose type is a generic class.
	 * @return	The PClassMapping of the generic class instance.
	 * @exception UnsupportedOperationException	Raised when the method is not
	 * 											supported by the PClassMapping.
	 */
	PClassMapping getGenClassMapping() throws UnsupportedOperationException;

	/**
	 * It gives access to the PClassMapping associated to a field whose type is
	 * a generic class.
	 * @param fn	The name of the generic class field.
	 * @return	The PClassMapping of the generic class instance.
	 * @exception UnsupportedOperationException	Raised when the method is not
	 * 											supported by the PClassMapping.
	 */
	PClassMapping getGenClassMapping(String fn)
			throws UnsupportedOperationException;

	/**
	 * It returns the Class/GenClass object representing the meta-information
	 * associated to the JORM class/generic class whose mapping is defined by
	 * this PClassMapping. It may be null if not requested at mapping time.
	 * @return	The Class/GenClass meta-object or null.
	 * @see	org.objectweb.jorm.api.PMapper#map
	 */
	MetaObject getMetaInfo();

	/**
	 * It yields the PBinder associated to this PClassMapping. As a PBinder
	 * itself, this PClassMapping delegates this behaviour to this PBinder.
	 * @return	The PBinder associated to this PClassMapping.
	 */
	PBinder getPBinder();

	/**
	 * It gives access to the mapper that has mapped the associated JORM class
	 * as defined by this PClassMapping.
	 * @return	The associated PMapper.
	 */
	PMapper getPMapper();

	/**
	 * It returns the naming context that manages the references contained by
	 * the generic class instance whose mapping is defined by this
	 * PClassMapping.
	 * @return	The naming context that manages references contained by a
	 * 			generic class instance.
	 * @exception	UnsupportedOperationException	It is raised when this
	 * 												PClassMapping does not map
	 * 												a generic class instance, or
	 * 												when the elements of this
	 * 												generic class instance are
	 * 												not references.
	 */
	PNameCoder getPNameCoder()
			throws UnsupportedOperationException;

	/**
	 * It returns the naming context that manages the reference associated to
	 * a field whose type is a JORM class.
	 * @return	The naming context that manages the reference defined by a
	 * 			field.
	 * @exception	UnsupportedOperationException	It is raised when no naming
	 * 												context is associated to
	 * 												the corresponding field.
	 */
	PNameCoder getPNameCoder(String fn)
			throws UnsupportedOperationException;

	/**
	 * It returns an iterator over all pname of persitent objects availlable in
	 * this class. When you use the returned iterator you must leave the
	 * connection opened.
	 * @param conn is the connection which permits to access to the support
	 * @return the iterator over PName objects
	 * @exception PExceptionProtocol	A problem occured during the request
	 * 									that is not related to I/O.
	 * @exception PExceptionIO	A problem occured during the request.
	 */
	Iterator getPNameIterator(Object conn) throws PException;

	/**
	 * Retrieves the PType of this mapped JORM class.
	 * @return The PTYpe.
	 */
	PType getPType();

	/**
	 * It specifies if the given kind of PMapper is supported by this
	 * PClassMapping.
	 * @param mappername	The name defining the PMapper kind.
	 * @return	true if this kind of mapper is supported.
	 */
	boolean isConform(String mappername);

	/**
	 * It assigns the PBinder associated to this PClassMapping. As a PBinder
	 * itself, this PClassMapping delegates this behaviour to this PBinder.
	 * @param pb	The PBinder associated to this PClassMapping.
	 * @exception	PExceptionTyping	It is raised when the structure of the
	 * 									the name that identifies the DSI for
	 * 									the PClassMapping is not supported by
	 * 									this PBinder that should be connected
	 * 									to this PClassMapping.
	 */
	void setPBinder(PBinder pb) throws PException;

	/**
	 * It permits to assign the PNameCoder of all the reference fields
	 * of the persitent class. The implementation of this method must call both
	 * method 'getPNameCoder' and 'isCodingOnly' of the specified PNCGetter.
	 * for each field which is a reference (to a class or a GenClass).
	 * @param rc is the callback object which knows the PNameCoder to use.
	 *
	 * @exception	PExceptionTyping	It is raised when the structure of the
	 * 									the names that reference the DSI stored
	 * 									within the generic class instance is not
	 * 									supported by the naming context returned
	 *                                  by the callBack method.
	 *
	 */
	void configureRefFields(ReferenceConfigurator rc) throws PException, UnsupportedOperationException;

	/**
	 *
	 */
	public interface ReferenceConfigurator {
		/**
		 * It retrieves the PNameCoder which must be associated to the field
		 * specified in parameter.
		 * @param sourceclassName is the class name which reference the object.
		 * @param refFieldName is the reference field.
		 * @param destclassName is the class name of the referenced Object.
		 * @return the PNameCoder to use for the specified field
		 */
		PNameCoder getPNameCoder(String sourceclassName,
								 String refFieldName,
								 String destclassName);

		/**
		 * It retrieves the PNameCoder which should manage the reference.
		 * @param refFieldName is the reference field.
		 * @param sourceclassName is the class name of the non generic class name
		 * which references a begin of the chain of generic Object.
		 * @param genClassNames is the list of the generic class name.
		 * @return the PNameCoder to use for the specified field
		 */
		PNameCoder getPNameCoder(String sourceclassName,
								 String refFieldName,
								 String[] genClassNames);

		/**
		 * It retrieves the PClassMapping which must be associated to the field
		 * specified in parameter.
		 * @param refFieldName is the reference field.
		 * @param sourceclassName is the class name of the non generic class name
		 * which references a begin of the chain of generic Object.
		 * @param genClassNames is the list of the generic class name.
		 * @return the PClassMapping to use for the specified field
		 */
		PClassMapping getGenClassMapping(String sourceclassName,
										 String refFieldName,
										 String[] genClassNames);

		/**
		 * It retrieves the PClassMapping which must be associated to the field
		 * specified in parameter.
		 * @param refFieldName is the reference field.
		 * @param sourceclassName is the class name of the non generic class name
		 * which references a begin of the chain of generic Object.
		 * @param genClassNames is the list of the generic class name.
		 * @param destclassName is the class name of the referenced Object.
		 * @return the PClassMapping to use for the specified field
		 */
		PClassMapping getGenClassMapping(String sourceclassName,
										 String refFieldName,
										 String[] genClassNames,
										 String destclassName);
	}
}
