/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.api;

import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.xml2mi.api.Parser;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.medor.eval.prefetch.api.PrefetchCache;

/**
 * The <b>PMapper</b> interface defines a JORM mapper as an object that mainly
 * maintains the set of JORM classes that it maps on a particular DS. It relies
 * on JCA as a means to access a DS.
 * It enables to access to the meta-information associated to JORM classes at
 * runtime.
 * Before being able to request the load of meta-information at mapping time,
 * the mapper should have been assigned a mapping name, a meta-information
 * manager, as well as a parser.
 * @author	P. Dechamboux
 */
public interface PMapper extends PMappingStructuresManager {
	final String PCLASSMAPPINGAPPENDER = "Mapping";
	final String PBINDINGAPPENDER = "Binding";
	final String JORMPROPFILE = "jorm.properties";

	/**
	 * Closes the given connection which has been allocated previously using
	 * getConnection.
	 * @param conn	The concerned connection.
	 */
	void closeConnection(Object conn) throws PException;

	/**
	 * Constructs a "mapped name" starting from a JORM class name. For example,
	 * "foo" class name produces "<getProjectName()>.foo", while "p1.p2.foo"
	 * produces "p1.p2.<getProjectName()>.foo".
	 * @param cn	The JORM class name.
	 */
	String cn2mn(String cn);

	/**
	 * Allocates a connection for accessing the underlying DS.
	 */
	Object getConnection() throws PException;

	/**
	 * Allocates a connection for accessing the underlying DS knowing additional
	 * context information.
	 * @param ctxt	Context information used to allocate the connection.
	 */
	Object getConnection(Object ctxt) throws PException;

	/**
	 * The name yielded by this method defines what kind of PMapper is used
	 * on a particular DS.
	 * @return	The name defining the PMapper kind.
	 */
	String getMapperName();

	/**
	 * Retrieves the PMappingStructuresManager associated with this mapper.
	 * @return The associated PMappingStructuresManager.
	 */
	PMappingStructuresManager getPMappingStructuresManager();

	/**
	 * It looks for the PClassMapping object associated to a JORM class within
	 * this PMapper.
	 * @return	The PClassMapping that maps the corresponding class, or null if
	 * 			none exists
	 */
	PClassMapping lookup(String classname);

	/**
	 * It registers a PClassMapping associated to a particular JORM class within
	 * this PMapper. It initializes the storage structures if necessary. It
	 * previously removes them if the "dsrem" flag is set. If the mapper is
	 * ready to manage the meta-information, it is loaded for this JORM class.
	 * @param pcm	The class mapping to add to this mapper.
	 * @exception PExceptionIO	It is raised when an I/O problem occurs within
	 * 							the underlying DS.
	 * @exception PExceptionProtocol	It is raised when a problem occurs
	 * 									during initialisation of the class
	 * 									mapping.
	 * @exception PExceptionTyping	It is raised when a typing problem occurs
	 * 								during initialisation of the class mapping.
	 */
	void map(PClassMapping pcm)
			throws PException;

	/**
	 * It registers a PClassMapping associated to a particular JORM class within
	 * this PMapper. It initializes the storage structures if necessary. It
	 * previously removes them if the "dsrem" flag is set. If the mapper is
	 * ready to manage the meta-information, it is loaded for this JORM class.
	 * @param conn	The connection to access to the underlying DS.
	 * @param pcm	The class mapping to add to this mapper.
	 * @exception PExceptionIO	It is raised when an I/O problem occurs within
	 * 							the underlying DS.
	 * @exception PExceptionProtocol	It is raised when a problem occurs
	 * 									during initialisation of the class
	 * 									mapping.
	 * @exception PExceptionTyping	It is raised when a typing problem occurs
	 * 								during initialisation of the class mapping.
	 */
	void map(Object conn, PClassMapping pcm)
			throws PException;

	/**
	 * It registers a PClassMapping associated to a particular JORM class within
	 * this PMapper. It initializes the storage structures if necessary. It
	 * previously removes them if the "dsrem" flag is set. It also loads the
	 * meta-information related to this class if the "loadmeta" flag is set.
	 * @param conn	The connection to access to the underlying DS.
	 * @param pcm		The class mapping to add to this mapper.
	 * @param loadmeta	It specifies that meta-information should be loaded for
	 * 					this class to map.
	 * @exception PExceptionIO	It is raised when an I/O problem occurs within
	 * 							the underlying DS.
	 * @exception PExceptionProtocol	It is raised when a problem occurs
	 * 									during initialisation of the class
	 * 									mapping.
	 * @exception PExceptionTyping	It is raised when a typing problem occurs
	 * 								during initialisation of the class mapping.
	 */
	void map(Object conn, PClassMapping pcm, boolean loadmeta)
			throws PException;

	/**
	 * Assigns a connection factory to this mapper.
	 * @exception PExceptionProtocol	It is raised if a connection factory
	 * 									has already been assigned.
	 */
	void setConnectionFactory(Object cf) throws PException;

	/**
	 * Returns the connection factory associated to this mapper.
	 */
	Object getConnectionFactory();

	/**
	 * Assigns a prefetch cache to this mapper. It is optional.
	 * @exception PExceptionProtocol	It is raised if a prefetch cache
	 * 									has already been assigned.
	 */
	void setPrefetchCache(PrefetchCache pc) throws PException;

	/**
	 * Returns the prefetch cache associated to this mapper. It can be null.
	 */
	PrefetchCache getPrefetchCache();

	/**
	 * It assigns the name of the mapper that is instanciated. It is mandatory
	 * to assign it at mapper initialization as the load of meta-information
	 * depends on it.
	 * @param mappername	The name of the mapper.
	 */
	void setMapperName(String mappername);

	/**
	 * It retrieves a meta-information Manager that may used by a parser in order
	 * to load the meta-information associated to JORM classes mapped within
	 * this PMapper.
	 * @return	The meta-information Manager assigned.
	 */
	Manager getMetaInfoManager();

	/**
	 * It returns a PTypeSpace that may be used to create PType
	 * associated to JORM classes mapped within this PMapper.
	 */
	PTypeSpace getPTypeSpace();

	/**
	 * Starts this mapper.
	 * @throws PException
	 */
	void start() throws PException;

	/**
	 * Stops this mapper.
	 * @throws PException
	 */
	void stop() throws PException;

	/**
	 * It unmaps a particular JORM class within this PMapper.
	 * @param classname	The name of the JORM class to be unmapped.
	 */
	void unmap(String classname) throws PException;

	/**
	 * Adds the given mapper listener to this mapper. If it already exists,
	 * does nothing.
	 * @param listener  The corresponding listener.
	 */
	void addMapperEventListener(PMapperListener listener);

	/**
	 * Removes the given mapper listener from this mapper. If it is not
	 * present among this list, does nothing.
	 * @param listener  The corresponding listener.
	 */
	void removeMapperEventListener(PMapperListener listener);

	/**
	 * Creates a new PClassMapping for a gen class. It instanciates the relevant
	 * class wrt the type of mapper.
	 * @return
	 */
	PClassMapping createGenClassMapping();
}
