/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.facility.naming.generator;

import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.naming.api.PName;

/**
 * Implements a long generator (LongGen) that produces new long ID merely by
 * incrementing a persistent long variable.
 * @author	P. Dechamboux
 */
public abstract class LongGenIncr
		implements LongGen, PAccessor, LongGenIncrAccessor, PBinding {
	private LongGenIncrMgr mgr;
	/**
	 * The next long ID to be delivered by next call to genId. This is
	 * also a persistent variable.
	 */
	private long nextId = 0;

	// IMPLEMENTATION OF METHODS FROM THE LongGen INTERFACE

	/**
	 * Initializes this long generator. Verifies if it already exists and if
	 * not, creates it.
	 * @param name	The name identifier associated with this long generator.
	 * @param pcm	The associated PClassMapping.
	 * @param mgr	The manager associated with this generator.
	 */
	public void init(String name, PClassMapping pcm, LongGenMgr mgr)
			throws PException {
		this.mgr = (LongGenIncrMgr) mgr;
		// Initializes the PBinding
		init(pcm);
		// Verifies if it already exists
		PName lgpn = pcm.getPBinder().decodeString(name);
		Object conn = mgr.getPMapper().getConnection();
		try {
			this.bind(lgpn);
			if (exist(conn)) {
				read(conn, this);
			} else {
				// Creates a new one and initializes it.
				export(conn, name);
				write(conn, this);
			}
		} catch (PException pe) {
			mgr.getPMapper().closeConnection(conn);
			throw pe;
		}
		mgr.getPMapper().closeConnection(conn);
	}

	/**
	 * Generates a new long identifier.
	 * @return	The new identifier.
	 */
	public synchronized long genId() throws PException {
		nextId++;
		Object conn = mgr.getPMapper().getConnection();
		try {
			this.write(conn, this);
		} catch (PException pe) {
			mgr.getPMapper().closeConnection(conn);
			throw pe;
		}
		mgr.getPMapper().closeConnection(conn);
		return nextId - 1;
	}

	// IMPLEMENTATION OF METHODS FROM THE PAccessor INTERFACE

	/**
	 * It returns the memory instance within which the persistent variables are
	 * defined.
	 * @return	The actual object containing memory persistent variables.
	 */
	public Object getMemoryInstance() {
		return this;
	}

	// IMPLEMENTATION OF METHODS FROM THE LongGenAccessor INTERFACE

	/**
	 * nextId getter.
	 */
	public long paGetNextId() {
		return nextId;
	}

	/**
	 * nextId setter.
	 */
	public void paSetNextId(long val) {
		nextId = val;
	}
}
