/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.facility.naming.polymorphid;

import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PExceptionNoDSI;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.facility.naming.generator.LongGen;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PExceptionExistingName;
import org.objectweb.perseus.cache.api.CacheEntry;
import org.objectweb.perseus.cache.api.CacheException;

/**
 * Implements a binder that manages polymorphic names. Such names are composed
 * of the identifier of the persistent class to which they belong (see classId
 * below), and of an object identifier within this binder, which is generated
 * by this binder. The classId information is persistent; thus, this class is
 * extended by ClassIdBinding generated by JORM using "ClassId.pd".
 * @author	P. Dechamboux
 */
public class PolymorphIdBinderInfo
		extends PolymorphIdBinder
		implements PAccessor, ClassIdAccessor, IdClassAccessor, PBinder {
	public static final String CLASSIDN =
			"org.objectweb.jorm.facility.naming.polymorphid.ClassId";
	public static final String IDCLASSN =
			"org.objectweb.jorm.facility.naming.polymorphid.IdClass";
	Long classId;
	String className;
	private PolymorphIdMgr mgr;
	private LongGen objectIdGen;

	/**
	 * Initializes this new binder. It either loads its persistent information
	 * if it has already been created, or it creates all this persistent
	 * information and immediatly stores them.
	 */
	void init(PMapper m, String cn, PolymorphIdMgr mgr) throws PException {
		Object conn = mgr.getPMapper().getConnection();
		try {
			init(m, cn, mgr, conn);
		} catch (PException e) {
			throw e;
		} finally {
			mgr.getPMapper().closeConnection(conn);
		}
	}

	/**
	 * Initializes this new binder. It either loads its persistent information
	 * if it has already been created, or it creates all this persistent
	 * information and immediatly stores them.
	 */
	void init(PMapper m, String cn, PolymorphIdMgr mgr, Object conn) throws PException {
		this.mgr = mgr;
		className = cn;
		PClassMapping pcm = m.lookup(CLASSIDN);
		PBinding binding = pcm.createPBinding();
		objectIdGen = mgr.getLongGenMgr().getLongGen(className);
		binding.bind(pcm.getPBinder().decodeString(className));
		Object connection = conn;
		if (connection == null)
			connection = mgr.getPMapper().getConnection();
		try {
			binding.read(connection, this);
		} catch (PExceptionNoDSI e) {
			binding.export(connection, className);
			classId = new Long(mgr.newClassId());
			binding.write(connection, this);
			PClassMapping pcm2 = m.lookup(IDCLASSN);
			PBinding binding2 = pcm2.createPBinding();
			binding2.bind(pcm2.getPBinder().decodeLong(classId.longValue()));
			if (!binding2.exist(connection)) {
				binding2.export(connection, classId);
				binding2.write(connection, this);
			}
			binding2.unbind();
		}
		binding.unbind();
	}

	/**
	 * It initializes the Binder with a specified identifier. The conn is used
	 * to search the class name associated to this id. If the class name is not
	 * found a PExceptionNaming is thrown.
	 * @param m is the mapper in which the class IdClass is mapped
	 * @param id is the class identifier
	 * @param mgr is the PolymorphIdMgr
	 * @param conn is the connection to access to the data support which stores
	 * the naming meta data.
	 * @throws PExceptionNaming when the identifier is associated to any
	 * clas name. And throws a PException if erros occurs during the loading
	 * of the IdClass object.
	 */
	void init(PMapper m, long id, PolymorphIdMgr mgr, Object conn)
			throws PException {
		Object connection = conn;
		if (connection == null)
			connection = mgr.getPMapper().getConnection();
		try {
			this.mgr = mgr;
			PClassMapping pcm = m.lookup(IDCLASSN);
			PBinding binding = pcm.createPBinding();
			binding.bind(pcm.getPBinder().decodeLong(id));
			try {
				binding.read(connection, this);
			} catch (PExceptionNoDSI e) {
				throw new PExceptionNaming("The binder " + id + " does not exist");
			} finally {
				binding.unbind();
			}
			objectIdGen = mgr.getLongGenMgr().getLongGen(className);
		} catch (PException e) {
			throw e;
		} finally {
			mgr.getPMapper().closeConnection(connection);
		}
	}


	// IMPLEMENTATION OF METHODS FROM THE PNameManager INTERFACE

	public String getClassName() {
		return className;
	}

	public void setPType(PType pt) {
		super.setPType(pt);
		mgr.binderTypeDef(this);
	}

	public PName export(Object conn, Object binding) throws PException {
		PBindingCtrl pb = (PBindingCtrl) binding;
		byte nextstate = PStateGraph.nextStatePBinding(pb.getStatus(),
													   PBinding.ACTION_EXPORT);
		if (nextstate == PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized operation");
		// Performs the transition
		PName res = new PolymorphIdPName(classId.longValue(),
										 objectIdGen.genId(), this);
		if (bindingcache != null) {
			synchronized (bindingcache) {
				if (bindingcache.lookup(res) != null) {
					throw new PExceptionExistingName(
							"[" + getClassName() + "]: an object has been already export with the same identifier");
				}
				try {
					CacheEntry ce = bindingcache.bind(res, binding);
					bindingcache.fix(ce);
				} catch (CacheException e) {
					throw new PException(e, "[" + getClassName() + "]: problem with cache management");
				}
			}
		}
		pb.setPName(res);
		pb.setStatus(nextstate);
		return res;
	}

	public PName export(Object conn, Object binding, Object ctxt)
			throws PException {
		throw new PExceptionNaming("Supports only auto-generated names (no need of a user context).");
	}

	// IMPLEMENTATION OF METHODS FROM THE PAccessor INTERFACE

	public Object getMemoryInstance() {
		return this;
	}

	// IMPLEMENTATION OF METHODS FROM THE PAccessorClassId INTERFACE

	//Accessors to the classId field
	public void paSetClassId(long val) throws PException {
		classId = new Long(val);
	}

	public long paGetClassId() throws PException {
		return classId.longValue();
	}

	// IMPLEMENTATION OF METHODS FROM THE PAccessorIdClass INTERFACE

	//Accessors to the classId field
	public void paSetClassName(String val) throws PException {
		className = val;
	}

	public String paGetClassName() throws PException {
		return className;
	}
}
