/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.facility.naming.rdbsequence;

import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.naming.api.PExceptionExistingName;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.lib.BasicPBinder;
import org.objectweb.jorm.mapper.rdb.adapter.RdbAdapter;
import org.objectweb.jorm.mapper.rdb.lib.PMapperRdb;
import org.objectweb.perseus.cache.api.CacheEntry;
import org.objectweb.perseus.cache.api.CacheException;
import org.objectweb.perseus.persistence.api.ConnectionHolder;
import org.objectweb.perseus.persistence.api.PersistenceException;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.ResultSet;

/**
 * This binder is specific to the rdb mapper.
 * @author S.Chassande-Barrioz
 */
public class RdbSequenceBinder extends BasicPBinder {

	private String seqName = null;
	private boolean notCreated = true;
	private String nextValRequest = null;

	public RdbSequenceBinder() {
		super();
		nullPName = new RdbSequencePName(this, RdbSequencePName.NULL_VALUE);
	}

	public RdbSequenceBinder(String seqName,
							 boolean createSequence) {
		this();
		notCreated = createSequence;
		this.seqName = seqName;
	}

	public boolean getCreateSequence() {
		return notCreated;
	}

	public void setCreateSequence(boolean notCreated) {
		this.notCreated = notCreated;
	}

	public String getSequenceName() {
		return seqName;
	}

	public void setSequenceName(String seqName) {
		this.seqName = seqName;
	}

	private void createSequence(Connection conn) throws PException {
		if (notCreated) {
			if (seqName == null) {
				throw new PException(
						"Impossible to allocate a new Sequence without its name (null).");
			}
			synchronized (this) {
				if (notCreated) {
					RdbAdapter adapter = ((PMapperRdb)
							getBinderClassMapping().getPMapper()).getRdbAdapter();
					nextValRequest = adapter.getNextValInSequence(seqName);
					Statement s = null;
					try {
						if (!adapter.existSequence(conn, seqName)) {
							s = conn.createStatement();
							s.execute(adapter.getCreateSequence(seqName));
						}
					} catch (SQLException e) {
						throw new PException(e,
											 "Impossible to create the sequence '"
											 + seqName + "'.");
					} finally {
						if (s != null) {
							try {
								s.close();
							} catch (SQLException e) {
							}
						}
					}
					notCreated = false;
				}
			}
		}
	}

	private long allocateId(Connection conn) throws PException {
		Statement s = null;
		ResultSet rs = null;
		if (nextValRequest == null) {
			RdbAdapter adapter = ((PMapperRdb)
					getBinderClassMapping().getPMapper()).getRdbAdapter();
			nextValRequest = adapter.getNextValInSequence(seqName);
		}
		try {
			s = conn.createStatement();
			rs = s.executeQuery(nextValRequest);
			if (rs.next()) {
				return rs.getLong(1);
			} else {
				throw new PExceptionIO("Strong case: The nextval operation does " +
									   "not return any value!");
			}
		} catch (SQLException e) {
			throw new PExceptionIO(e, "Impossible to allocate a new identifier"
									  + " on the sequence: " + seqName);
		} finally {
			if (rs != null) {
				try {
					rs.close();
				} catch (SQLException e) {
				}
			}
			if (s != null) {
				try {
					s.close();
				} catch (SQLException e) {
				}
			}
		}
	}

	// IMPLEMENTATION OF THE METHOD OF BasicPNamingContext INTERFACE
	
	public PName resolve(Object conn, PName pn) throws PException {
		if (pn == null) {
			throw new PExceptionNaming("[" + getClassName()
									   + "]: this pname is null");
		}
		if (!pn.getPNameManager().equals(this))
			throw new PExceptionNaming("[" + getClassName()
									   + "]: this pname is not valid in this binder");
		return pn;
	}

	public PName export(Object conn, Object infoitem, Object hints)
			throws PException {
		if (hints instanceof RdbSequencePName) {
			//Binder role
			PBindingCtrl pb = (PBindingCtrl) infoitem;
			byte nextstate = PStateGraph.nextStatePBinding(pb.getStatus(),
														   PBinding.ACTION_EXPORT);
			if (nextstate == PBinding.LIFECYCLE_ERROR)
				throw new PExceptionProtocol("Unauthorized operation");

			PName pn = new RdbSequencePName(this, ((RdbSequencePName) hints).value);
			if (cache != null) {
				synchronized (cache) {
					if (cache.lookup(pn) != null) {
						throw new PExceptionExistingName("[" + getClassName()
														 + "]: an object has been already export with the same identifier");
					}
					try {
						System.out.println("fix");
						cache.fix(cache.bind(pn, pb));
					} catch (CacheException e) {
						throw new PException(e, "[" + getClassName()
												+ "]: problem with cache management");
					}
				}
			}
			pb.setPName(pn);
			pb.setStatus(nextstate);
			return pn;
		} else {
			if (seqName == null && (hints instanceof String)) {
				seqName = (String) hints;
			}
			return export(conn, infoitem);
		}
	}

	public PName export(Object c, Object en) throws PException {
		if (en == null) {
			throw new PExceptionNaming("[" + getClassName() + "]: cannot export null!");
		}
		if (en instanceof PName) {
			//Naming context role
			if (((PName) en).getPNameManager() == this) {
				return (PName) en;
			} else {
				return new RdbSequencePName(this, en);
			}
		}

		//Binder role
		PBindingCtrl pb = (PBindingCtrl) en;
		byte nextstate = PStateGraph.nextStatePBinding(pb.getStatus(),
													   PBinding.ACTION_EXPORT);
		if (nextstate == PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized operation");

		PName pn = null;
		try {
			Connection conn;
			if (c == null) {
				conn = (Connection) getBinderClassMapping().getPMapper().getConnection();
			} else if (c instanceof Connection) {
				conn = (Connection) c;
			} else if (c instanceof ConnectionHolder) {
				conn = (Connection) ((ConnectionHolder) c).getCHConnectionForWrite();
			} else {
				throw new PException("Impossible to fetch a connection: unknown connection type: " + c);
			}
			if (notCreated) {
				createSequence(conn);
			}

			pn = new RdbSequencePName(this, allocateId(conn));
			if (c == null) {
				getBinderClassMapping().getPMapper().closeConnection(conn);
			} else if (c instanceof ConnectionHolder) {
				((ConnectionHolder) c).releaseCHConnection();
			}
		} catch (PersistenceException e) {
			throw new PException(e, "Problem with Perseus connection management.");
		}

		if (cache != null) {
			synchronized (cache) {
				if (cache.lookup(pn) != null) {
					throw new PExceptionExistingName("[" + getClassName()
													 + "]: an object has been already export with the same identifier");
				}
				try {
					cache.fix(cache.bind(pn, pb));
				} catch (CacheException e) {
					throw new PException(e, "[" + getClassName()
											+ "]: problem with cache management");
				}
			}
		}
		pb.setPName(pn);
		pb.setStatus(nextstate);
		return pn;
	}

	public void unexport(Object conn, PName pn) throws PException {
		if (pn.isNull()) {
			throw new PExceptionProtocol("[" + getClassName() + "]: cannot unexport with a null pname.");
		}
		if (cache != null) {
			PBindingCtrl pb = null;
			CacheEntry ce = cache.lookup(pn);
			if (ce != null) {
				pb = (PBindingCtrl) ce.getCeObject();
			}
			if (pb == null) {
				throw new PExceptionProtocol("Unauthorized operation: No Pbinding found");
			}
			byte nextstate = PStateGraph.nextStatePBinding(
					pb.getStatus(), PBinding.ACTION_UNEXPORT);
			if (nextstate == PBinding.LIFECYCLE_ERROR) {
				throw new PExceptionProtocol("Unauthorized operation: status =" + pb.getStatus());
			}
			pb.setStatus(nextstate);
		} else {
			throw new PExceptionProtocol("Unauthorized operation: No Pbinding can be found without a CacheManager");
		}
	}

	public void unexport(Object conn, PName pn, Object hints) throws PException {
		if (pn.isNull()) {
			throw new PExceptionProtocol("[" + getClassName() + "]: cannot unexport with a null pname.");
		}
		PBindingCtrl pb = null;
		if (hints instanceof PBindingCtrl) {
			pb = (PBindingCtrl) hints;
		} else if (cache != null) {
			CacheEntry ce = cache.lookup(pn);
			if (ce != null) {
				pb = (PBindingCtrl) ce.getCeObject();
			}
		}
		if (pb == null) {
			throw new PExceptionProtocol("Unauthorized operation: No Pbinding found");
		}
		byte nextstate = PStateGraph.nextStatePBinding(
				pb.getStatus(), PBinding.ACTION_UNEXPORT);
		if (nextstate == PBinding.LIFECYCLE_ERROR) {
			throw new PExceptionProtocol("Unauthorized operation: status =" + pb.getStatus());
		}
		pb.setStatus(nextstate);
	}

	public long encodeLong(PName pn) throws PExceptionNaming, UnsupportedOperationException {
		return ((RdbSequencePName) pn).value;
	}

	public PName decodeLong(long en) throws PExceptionNaming, UnsupportedOperationException {
		return new RdbSequencePName(this, en);
	}

	public PName decodeString(String en) throws PExceptionNaming {
		try {
			return new RdbSequencePName(this, Long.parseLong(en));
		} catch (NumberFormatException e) {
			throw new PExceptionNaming("[" + getClassName()
									   + "]: Impossible to decode the String to long: " + en);
		}
	}

	public boolean codingSupported(int codingtype) {
		switch (codingtype) {
		case CTLONG:
		case CTSTRING:
			return true;
		default:
			return false;
		}
	}

	public String encodeString(PName pn) throws PExceptionNaming {
		return "" + ((RdbSequencePName) pn).value;
	}
}
