/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.genclass.lib;

import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;

import java.util.ArrayList;

/**
 * This abstract class provides all the generic code for PBinding associated
 * with JORM generic classes. It implements all methods except "exist", "read",
 * and "write" (indeed, all I/O methods are DS-specific).
 * @author	P. Dechamboux
 */

public abstract class GenClassBinding implements PBindingCtrl {
	/**
	 * The current state of this PBinding object.
	 */
	protected byte status;

	/**
	 * The PClassMapping that manages this GenClass PBinding.
	 */
	protected GenClassMapping genClassMapping;

	/**
	 * The PName of this PBinding object, which gives access to the relevant
	 * DSI. This should always have a value (never null) and it should have
	 * a "null" PName value when unbound.
	 */
	protected PName pName;

	/**
	 * This fields
	 */
	protected ArrayList indexes = null;

	/**
	 * It contructs
	 */
	public GenClassBinding() {
	}

	/**
	 * It contructs
	 */
	public GenClassBinding(GenClassMapping mapping) throws PExceptionProtocol {
		init(mapping);
	}

	// IMPLEMENTATION OF METHODS FROM THE PBinding INTERFACE

	/**
	 * It assigns a DSI represented by the PName passed as a parameter to this
	 * PBinding. This thus changes the link to the data store instance if one
	 * has been already assigned. The state either changed to
	 * LIFECYCLE_NOACCESSOR and LIFECYCLE_ACTIVEFORIO.
	 * @param pn    The new PName to associate to this binding. This name
	 *              must belong to the binder associated to this binding.
	 * @exception PExceptionNaming	It is raised when there is a problem for the
	 * 								associated binder to manage this PName.
	 */
	public void bind(PName pn) throws PException {
		genClassMapping.getPBinder().bind(pn, this);
	}

	/**
	 * It creates a new PName for this binding, the binding state
	 * is normally changed to LIFECYCLE_NEWTOWRITE, and the link to a data
	 * store instance if it existed is removed.
	 * @param conn	The connection that can be used to refer to the DS.
	 * @return	The new PName assigned to this binding.
	 * @exception PExceptionIO	It is raised when a problem occured while
	 * 							accessing to the DS.
	 * @exception PExceptionNaming	It is raised when the associated binder
	 * 								does not manage to create a new name.
	 * @exception PExceptionProtocol	It is raised when it is called with a
	 * 									state different from the following ones:
	 * 									LIFECYCLE_ACTIVEFORIO
	 * 									LIFECYCLE_DELTOWRITE
	 * 									LIFECYCLE_NEWTOWRITE
	 * 									LIFECYCLE_NOACC
	 */
	public PName export(Object conn) throws PException {
		return genClassMapping.getPBinder().export(conn, this);
	}

	public PName export(Object conn, Object hints) throws PException {
		return genClassMapping.getPBinder().export(conn, this, hints);
	}

	/**
	 * It gives access to the PClassMapping which manages this binding.
	 * @return	The PClassMapping which creates this binding.
	 */
	public PClassMapping getPClassMapping() {
		return genClassMapping;
	}

	/**
	 * It gives access to the PName, which designate a particular DSI,
	 * associated to this binding. It returns <code>null</code> if the binding
	 * has not been given a PName using a bind or export method.
	 * @return	The PName associated to this binding.
	 */
	public PName getPName() {
		return pName;
	}

	/**
	 * It yields the current status associated with this PBinding.
	 * @return	The corresponding status.
	 */
	public byte getStatus() {
		return status;
	}

	/**
	 * It initialises a PBinding.
	 * @param pcm	The PBinding to initialise.
	 * @exception PExceptionProtocol	This PBinding cannot be managed by this
	 * 									PClassMapping.
	 */
	public void init(PClassMapping pcm) throws PExceptionProtocol {
		if (pcm == null) {
			genClassMapping = null;
			return;
		}
		if (!(pcm instanceof GenClassMapping))
			throw new PExceptionProtocol(
					"Wrong PClassMapping associated with this GenClass PBinding: " + pcm);
		genClassMapping = (GenClassMapping) pcm;
		pName = genClassMapping.getPBinder().getNull();
		status = PBinding.LIFECYCLE_NOTBOUND;
	}

	/**
	 * It remove the association between a PName and a PBinding. No more PName
	 * is associated with this binding (it has no more link with a DSI).
	 * @exception PExceptionNaming	It is raised when there is a problem for the
	 * 								associated binder to manage this PName.
	 */
	public void unbind() throws PException {
		genClassMapping.getPBinder().unbind(this);
	}

	/**
	 * It prepares this PBinding to remove the data store instance from the
	 * data store. In order to actually remove this DSI, a call to write must
	 * follow.
	 * @param conn	The connection that can be used to refer to the DS.
	 * @exception PExceptionNaming	It is raised when there is a problem for the
	 * 								associated binder to manage this PName.
	 * @exception PExceptionProtocol	It is raised when it is called with a
	 * 									state different from the following ones:
	 * 									LIFECYCLE_ACTIVEFORIO
	 * 									LIFECYCLE_DELTOWRITE
	 * 									LIFECYCLE_NEWTOWRITE
	 */
	public void unexport(Object conn) throws PException {
		byte nextstate = PStateGraph.nextStatePBinding(
				status, PBinding.ACTION_UNEXPORT);
		if (nextstate == PBinding.LIFECYCLE_ERROR) {
			throw new PExceptionProtocol("Unauthorized operation");
		}
		status = nextstate;
	}

	/**
	 * It reads the data store instance designated by the PName of this
	 * binding and transfers each attribute value to the accessor object.
	 * This read method should be used when prefetching is requested.
	 * @param conn	The connection that can be used to refer to the DS.
	 * @param pa	The PAccessor used to access memory variables.
	 * @param txctx The transaction context that can be used in case of
	 *              prefetching.
	 * @exception PExceptionIO	It is raised when a problem occured while
	 * 							accessing to the DS.
	 * @exception PExceptionNaming	It is raised when there is a problem for the
	 * 								associated binder to manage this PName.
	 * @exception PExceptionProtocol	It is raised when it is called with a
	 * 									state different from the following ones:
	 * 									LIFECYCLE_ACTIVEFORIO
	 */
	public void read(Object conn, PAccessor pa, Object txctx) throws PException {
		// No prefetching for generic classes: delegate to normal read!!
		read(conn, pa);
	}

	// IMPLEMENTATION OF METHODS FROM THE PBindingCtrl INTERFACE

	/**
	 * It assigns the current PName associated with this PBinding.
	 */
	public void setPName(PName pn) {
		pName = pn;
	}

	/**
	 * It assigns the current status associated with this PBinding.
	 */
	public void setStatus(byte s) {
		status = s;
	}
}
