/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.generator.lib;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.context.Context;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.compiler.api.PExceptionCompiler;
import org.objectweb.jorm.compiler.lib.JormCompilerParameter;
import org.objectweb.jorm.metainfo.api.CompositeName;
import org.objectweb.jorm.metainfo.api.ScalarField;
import org.objectweb.jorm.metainfo.api.Package;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.util.io.api.TargetHolder;
import org.objectweb.util.monolog.api.BasicLevel;

import java.io.File;
import java.io.FileWriter;
import java.util.Iterator;

/**
 * This class is a generator of XXXBinder objects. It is generic either the
 * mapper type. This generator use the velocity tools. The used template is
 * Binder.vm . Associated to this template this generator builds a velocity
 * context which contains the following information:<br/>
 * <table border=1>
 * 	<tr><td>Key</td><td>Value</td></tr>
 * 	<tr><td>"compositename"</td><td>The reference to the compositename meta object</td></tr>
 * 	<tr>
 * 		<td>"tools"</td>
 * 		<td>The reference to the <a href="CommonHelper.html">CommonHelper<a/></td>
 * </tr>
 * 	<tr>
 * 		<td>"header"</td>
 * 		<td>The file name of the common template which contains the header of
 * 			the generate files.</td>
 * </tr>
 * </table>
 */
public class BinderGenerator extends CommonGenerator {
	public CommonHelper helper;

	/**
	 * This method generates a XPBinder file corresponding to the co
	 * parameter in the directory parameter.
	 * @param co		The meta object interface.
	 * @param holder	The target holder which allows to create files.
	 * @param cp		This parameter permits to reach the compilation parameters.
	 */
	public void generate(CompositeName co,
						 TargetHolder holder,
						 JormCompilerParameter cp) throws PException {
		// Calculate fileName
		String packName = ((Package) co.getParent()).getName();
		String fileName = co.getName() + "Binder";
		if (packName != null && packName.length() > 0) {
			fileName = packName + "." + fileName;
			fileName = fileName.replace('.', File.separatorChar);
		}
		logger.log(BasicLevel.DEBUG, "Generate the " + fileName + " Binder");
		try {
			// Fetch a block with the fileName
			FileWriter fw = holder.getFileWriter(fileName + ".java");
			Context ctx = new VelocityContext();
			ctx.put("compositename", co);
			helper = new CommonHelper();
			helper.setLogger(logger);
			ctx.put("tools", helper);
			ctx.put("bindertools", this);
			ctx.put("header", GEN_TEMPLATE_DIR + "Header.vm");
			Iterator it = co.iterateAllField();
			PrimitiveElement firstField = (PrimitiveElement) it.next();
			//If the composite name is composed by an unique field that the type is
			// different to String, specify a uniqueField in order to support the
			// direct coding. The String coding is always supported !!
			if (!it.hasNext()) {
				ctx.put("uniqueField", firstField);
				if (firstField.getType().getTypeCode() != PType.TYPECODE_STRING) {
					ctx.put("defaultStringEncoding", "toto");
				}
			} else {
				ctx.put("defaultStringEncoding", "toto");

			}
			velocityEngine.getTemplate(GEN_TEMPLATE_DIR + "Binder.vm").merge(ctx, fw);
			fw.flush();
			fw.close();
		} catch (Exception e) {
			throw new PExceptionCompiler(e, "Problem while generating PBinder.");
		}
	}

	public boolean containsObject(CompositeName cn) throws PException {
		for (Iterator it = cn.iterateAllField(); it.hasNext();) {
			if (helper.canBeNullValue(
					((ScalarField) it.next()).getType())) {
				return true;
			}
		}
		return false;
	}

	public boolean containsPrimitive(CompositeName cn) throws PException {
		for (Iterator it = cn.iterateAllField(); it.hasNext();) {
			if (!helper.canBeNullValue(
					((ScalarField) it.next()).getType())) {
				return true;
			}
		}
		return false;
	}

	public String getNullValue(PrimitiveElement pe) {
		if (helper.isObjectType(pe.getType())) {
			return "null";
		} else if (pe.getType().getTypeCode() == PType.TYPECODE_CHAR) {
			return "'0'";
		} else {
			return "(" + pe.getType().getJavaName() + ") -1";
		}
	}

	public String getPNameFromSpecificPNG(CompositeName cn,
										  String pngVar,
										  String ctxVar) throws PException {
		String result = "new " + cn.getName() + "PName(";
		Iterator fs = cn.iterateAllField();
		while (fs.hasNext()) {
			ScalarField f = (ScalarField) fs.next();
			result += "((" + cn.getName() + "PNG)" + pngVar + ").pnGet"
					+ helper.upperFL(f.getName()) + "(" + ctxVar + "),";
		}
		return result + " this)";
	}

	public String getPNameFromGenericPNG(CompositeName cn,
										 String pngVar,
										 String ctxVar) throws PException {
		String result = "new " + cn.getName() + "PName(";
		Iterator fs = cn.iterateAllField();
		while (fs.hasNext()) {
			ScalarField f = (ScalarField) fs.next();
			result += "((PNameGetter)" + pngVar + ")."
					+ helper.getPNameGetterGetFunction(f.getType()) + "(\""
					+ f.getName() + "\", " + ctxVar + "),";
		}
		return result + " this)";
	}

	public String getTestNullValues(CompositeName cn, boolean generic, String ctx)
			throws PException {
		StringBuffer res = new StringBuffer("");
		String sep = "";
		Iterator fs = cn.iterateAllField();
		while (fs.hasNext()) {
			ScalarField f = (ScalarField) fs.next();
			res.append(sep);
			sep = " && ";
			res.append("(");
			String v1;
			if (generic) {
				v1 = "png." + helper.getPNameGetterGetFunction(f.getType()) + "(\""
					+ f.getName() + "\", " + ctx + ")";
			} else {
				v1 = "png.pnGet" + helper.upperFL(f.getName() + "(" + ctx + ")");
			}
			res.append(helper.comparePE(v1, "jorm" + f.getName(), f.getType()));
			res.append(")");
		}
		return res.toString();
	}

	public String getCoderSetter(PType t) {
		if (t.getTypeCode() == PType.TYPECODE_BYTEARRAY) {
			return "putByteArray(";
		} else {
			return "put" + getCoderName(t) + "(";
		}
	}

	public String getCoderGetter(PType t) {
		if (t.getTypeCode() == PType.TYPECODE_BYTEARRAY) {
			return "getByteArray()";
		} else {
			return "get" + getCoderName(t) + "()";
		}
	}

	public String getCoderName(PType t) {
		switch (t.getTypeCode()) {
		case PType.TYPECODE_BYTE:
			return "Byte";
		case PType.TYPECODE_OBJBYTE:
			return "Obyte";
		case PType.TYPECODE_CHAR:
			return "Char";
		case PType.TYPECODE_OBJCHAR:
			return "Ochar";
		case PType.TYPECODE_DATE:
			return "Date";
		case PType.TYPECODE_BIGDECIMAL:
			return "BigDecimal";
		case PType.TYPECODE_BIGINTEGER:
			return "BigInteger";
		case PType.TYPECODE_SHORT:
			return "Short";
		case PType.TYPECODE_OBJSHORT:
			return "Oshort";
		case PType.TYPECODE_INT:
			return "Int";
		case PType.TYPECODE_OBJINT:
			return "Oint";
		case PType.TYPECODE_LONG:
			return "Long";
		case PType.TYPECODE_OBJLONG:
			return "Olong";
		case PType.TYPECODE_STRING:
			return "String";
		case PType.TYPECODE_BYTEARRAY:
			return "";
		case PType.TYPECODE_CHARARRAY:
			return "CharArray";
		}
		return null;
	}

	public String getCTDeclaration(PType t) {
		switch (t.getTypeCode()) {
		case PType.TYPECODE_CHAR:
			return "PNamingContext.CTCHAR";
		case PType.TYPECODE_OBJCHAR:
			return "PNamingContext.CTOCHAR";

		case PType.TYPECODE_BYTE:
			return "PNamingContext.CTBYTE";
		case PType.TYPECODE_OBJBYTE:
			return "PNamingContext.CTOBYTE";

		case PType.TYPECODE_SHORT:
			return "PNamingContext.CTSHORT";
		case PType.TYPECODE_OBJSHORT:
			return "PNamingContext.CTOSHORT";

		case PType.TYPECODE_INT:
			return "PNamingContext.CTINT";
		case PType.TYPECODE_OBJINT:
			return "PNamingContext.CTOINT";

		case PType.TYPECODE_LONG:
			return "PNamingContext.CTLONG";
		case PType.TYPECODE_OBJLONG:
			return "PNamingContext.CTOLONG";

		case PType.TYPECODE_STRING:
			return "PNamingContext.CTSTRING";

		case PType.TYPECODE_BYTEARRAY:
			return "PNamingContext.CTBYTEARRAY";
		case PType.TYPECODE_CHARARRAY:
			return "PNamingContext.CTCHARARRAY";
		case PType.TYPECODE_DATE:
			return "PNamingContext.CTDATE";
		case PType.TYPECODE_BIGDECIMAL:
			return "PNamingContext.CTBIGDECIMAL";
		case PType.TYPECODE_BIGINTEGER:
			return "PNamingContext.CTBIGINTEGER";
		}
		return null;
	}
}
