/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.generator.lib;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.context.Context;
import org.objectweb.jorm.compiler.api.PExceptionCompiler;
import org.objectweb.jorm.compiler.lib.JormCompilerParameter;
import org.objectweb.jorm.lib.JormConfigurator;
import org.objectweb.jorm.generator.api.MOP;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.Package;
import org.objectweb.jorm.metainfo.api.Mapping;
import org.objectweb.jorm.metainfo.api.ClassProject;
import org.objectweb.jorm.util.io.api.TargetHolder;
import org.objectweb.jorm.api.PException;
import org.objectweb.util.monolog.api.BasicLevel;

import java.io.File;
import java.io.FileWriter;
import java.util.Iterator;
import java.util.HashMap;
import java.util.Map;

/**
 * This class is a generator of XXXBinding objects. It is generic either the
 * mapper type. This generator use the velocity tools. The used template is
 * Binding.vm . Associated to this template this
 * generator builds a velocity context which contains the following
 * information:<br/>
 * <table border=1>
 * 	<tr><td>Key</td><td>Value</td></tr>
 * 	<tr><td>"class"</td><td>The reference to the Class meta object</td></tr>
 * 	<tr>
 * 		<td>"tools"</td>
 * 		<td>The reference to the <a href="CommonHelper.html">CommonHelper<a/></td>
 * </tr>
 * 	<tr>
 * 		<td>"header"</td>
 * 		<td>The file name of the common template which contains the header of
 * 			the generate files.</td>
 * </tr>
 * 	<tr>
 * 		<td>"bindingTools"</td>
 * 		<td>The reference to current instance of BindingGenerator.</td>
 * </tr>
 * 	<tr>
 * 		<td>"cparam"</td>
 * 		<td>The reference to a CompilerParameter instance. This instance
 * 		describes the user compilation parameter.</td>
 * </tr>
 * </table>
 *
 * As a XXXBinding class is specific to a mapper, the Binding.vm template uses
 * externals macros which must be defined as an external template library in
 * the velocity tools.
 */
public class BindingGenerator extends CommonGenerator {
	/**
	 * The mapper generation specializations (some MOP objects)
	 */
	protected HashMap bmops = new HashMap();

	/**
	 * This constrcutor permits to specify which BindingMOP must be used
	 */
	public BindingGenerator(JormConfigurator jcc, JormCompilerParameter cp) throws PException {
		super();
		Iterator it = jcc.knownMappers();
		while (it.hasNext()) {
			String mn = (String) it.next();
			bmops.put(mn, jcc.getMOPFactory(mn).createBindingMOP(cp, mn));
		}
	}

	public String[][] getTemplateLibraries() {
		String[][] res = new String[bmops.size()][];
		Iterator it = bmops.values().iterator();
		int i = 0;
		while (it.hasNext()) {
			res[i] = ((MOP) it.next()).getTemplateLibraries();
			i++;
		}
		return res;
	}

	/**
	 * This method generates a XPBinding file corresponding to the pod
	 * parameter in the directory parameter
	 * @param co		the class meta object interface
	 * @param holder	The target holder which allows to create files
	 * @param cp		This parameter permits to reach the compilation parameters
	 * @param genctxts  The context variables used by the velocity template engine.
	 */
	public void generate(Class co, TargetHolder holder,
						 JormCompilerParameter cp, JormConfigurator jcc,
						 Map genctxts) throws PException {
		ClassProject cpr = null;
		Iterator it = co.getClassProjects().iterator();
		while (it.hasNext()) {
			cpr = (ClassProject) it.next();
			if (cpr.getProjectName().equals(cp.projectName)) {
				break;
			} else {
				cpr = null;
			}
		}
		if (cpr == null) {
			throw new PExceptionCompiler("Cannot find the relevant project!");
		}
		it = cpr.getMappings().iterator();
		while (it.hasNext()) {
			Mapping mo = (Mapping) it.next();
			Iterator it2 = jcc.getSubMappers(mo.getMapperName()).iterator();
			//Iterator it2 = cp.getSubMappers(mo.getMapperName()).iterator();
			if (it2.hasNext()) {
				while (it2.hasNext()) {
					String submap = (String) it2.next();
					String mn = mo.getMapperName()
							+ (submap.equals("generic") ? "" : "." + submap);
					generateWithMapper(mn, co, holder, cp, genctxts, mo);
				}
			} else {
				generateWithMapper(mo.getMapperName(), co, holder, cp, genctxts, mo);
			}
		}
	}

	private void generateWithMapper(String mn, Class co, TargetHolder holder,
									JormCompilerParameter cp, Map genctxts,
									Mapping mo) throws PException {
		String pack = ((Package) co.getParent()).getName();
		if (cp.generatedWithMapperPackage) {
			pack = ("".equals(pack) ? mn : pack + "." + mn);
		}
		// File name
		String fileName = co.getName() + "Binding";
		if (pack.length() > 0) {
			fileName = (pack + '.').replace('.', File.separatorChar) + fileName;
		}
		logger.log(BasicLevel.DEBUG, "Generate the " + fileName + " class");
		// Fetch a block with the fileName
		FileWriter fw = null;
		try {
			MOP bmop = (MOP) bmops.get(mo.getMapperName());
			fw = holder.getFileWriter(fileName + ".java");
			Context ctx = new VelocityContext();
			ctx.put("class", co);
			ctx.put("mapping", mo);
			ctx.put("mappername", mn);
			if (pack.length() > 0) {
				ctx.put("package", pack);
			}
			ctx.put("cparam", cp);
			CommonHelper ch = new CommonHelper();
			ctx.put("containsCharArrayField", new Boolean(ch.containsCharArrayField(mo)));
			ch.setLogger(logger);
			ctx.put("tools", ch);
			ctx.put("header", GEN_TEMPLATE_DIR + "Header.vm");
			ctx.put("bindingTools", this);
			ctx.put("genInfos", genctxts.get(mo.getMapperName()));
			bmop.initContext(ctx);
			velocityEngine.getTemplate(GEN_TEMPLATE_DIR + "Binding.vm").merge(ctx, fw);
			fw.flush();
			fw.close();
		} catch (Exception e) {
			e.printStackTrace();
			throw new PExceptionCompiler(e,
										 "Problem while writing Binding java file: " + e.getMessage());
		}
	}
}
