/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.generator.lib;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.context.Context;
import org.objectweb.jorm.compiler.api.PExceptionCompiler;
import org.objectweb.jorm.compiler.lib.JormCompilerParameter;
import org.objectweb.jorm.lib.JormConfigurator;
import org.objectweb.jorm.generator.api.MOP;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.GenClass;
import org.objectweb.jorm.metainfo.api.Package;
import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jorm.metainfo.api.TypedElement;
import org.objectweb.jorm.metainfo.api.Mapping;
import org.objectweb.jorm.metainfo.api.ClassProject;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.util.io.api.TargetHolder;
import org.objectweb.util.monolog.api.BasicLevel;

import java.io.File;
import java.io.FileWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Map;

/**
 * This class is a generator of XPMapping objects. It is generic either the
 * mapper type. This generator use the velocity tools. The used template is
 * Mapping.vm . Associated to this template, this
 * generator builds a velocity context which contains the following
 * information:<br/>
 * <table border=1>
 * 	<tr><td>Key</td><td>Value</td></tr>
 * 	<tr><td>"class"</td><td>The reference to the Class meta object</td></tr>
 * 	<tr>
 * 		<td>"tools"</td>
 * 		<td>The reference to the <a href="CommonHelper.html">CommonHelper<a/></td>
 * </tr>
 * 	<tr>
 * 		<td>"header"</td>
 * 		<td>The file name of the common template which contains the header of
 * 			the generate files.</td>
 * </tr>
 * 	<tr>
 * 		<td>"mappingTools"</td>
 * 		<td>The reference to current instance of MappingGenerator.</td>
 * </tr>
 * 	<tr>
 * 		<td>"cparam"</td>
 * 		<td>The reference to a CompilerParameter instance. This instance
 * 		describes the user compilation parameter.</td>
 * </tr>
 * </table>
 *
 * As a XXXMapping class is specific to a mapper, the Mapping.vm template uses
 * externals macros which must be defined as an external template library in
 * the velocity tools.
 */
public class MappingGenerator extends CommonGenerator {
	/**
	 * The mapper generation specializations (some MOP objects)
	 */
	protected HashMap mmops = new HashMap();

	/**
	 * This constrcutor permits to specify which BindingMOP must be used
	 */
	public MappingGenerator(JormConfigurator jcc, JormCompilerParameter cp) throws PException {
		super();
		Iterator it = jcc.knownMappers();
		while (it.hasNext()) {
			String mn = (String) it.next();
			mmops.put(mn, jcc.getMOPFactory(mn).createMappingMOP(cp, mn));
		}
	}

	public String[][] getTemplateLibraries() {
		String[][] res = new String[mmops.size()][];
		Iterator it = mmops.values().iterator();
		int i = 0;
		while (it.hasNext()) {
			res[i] = ((MOP) it.next()).getTemplateLibraries();
			i++;
		}
		return res;
	}

	/**
	 * This method generates a XXXPMapping file corresponding to the pod
	 * parameter in the directory parameter
	 * @param co		the meta object interface
	 * @param holder	The target holder which allows to create files
	 * @param cp		This parameter permits to reach the compilation parameters
	 * @param genctxts  The context variables used by the velocity template engine.
	 */
	public void generate(Class co, TargetHolder holder,
						 JormCompilerParameter cp, JormConfigurator jcc,
						 Map genctxts) throws PException {
		ClassProject cpr = null;
		Iterator it = co.getClassProjects().iterator();
		while (it.hasNext()) {
			cpr = (ClassProject) it.next();
			if (cpr.getProjectName().equals(cp.projectName)) {
				break;
			} else {
				cpr = null;
			}
		}
		if (cpr == null) {
			throw new PExceptionCompiler("Cannot find the relevant project!");
		}
		it = cpr.getMappings().iterator();
		while (it.hasNext()) {
			Mapping mo = (Mapping) it.next();
			Iterator it2 = jcc.getSubMappers(mo.getMapperName()).iterator();
			//cp.getSubMappers(mo.getMapperName()).iterator();
			if (it2.hasNext()) {
				do {
					String submap = (String) it2.next();
					String mn = mo.getMapperName()
							+ (submap.equals("generic") ? "" : "." + submap);
					generateWithMapper(mn, co, holder, cp, genctxts, mo);
				} while (it2.hasNext());
			} else {
				generateWithMapper(mo.getMapperName(), co, holder, cp, genctxts, mo);
			}
		}
	}

	private void generateWithMapper(String mn, Class co, TargetHolder holder,
									JormCompilerParameter cp, Map genctxts,
									Mapping mo) throws PException {
		String pack = ((Package) co.getParent()).getName();
		if (cp.generatedWithMapperPackage) {
			pack = ("".equals(pack) ? mn : pack + "." + mn);
		}
		// File name
		String fileName = co.getName() + PMapper.PCLASSMAPPINGAPPENDER;
		if (pack.length() > 0) {
			fileName = (pack + '.').replace('.', File.separatorChar) + fileName;
		}
		logger.log(BasicLevel.DEBUG, "Generate the " + fileName + " class");
		try {
			MOP mmop = (MOP) mmops.get(mo.getMapperName());
			// Fetch a block with the fileName
			FileWriter fw = holder.getFileWriter(fileName + ".java");
			Context ctx = new VelocityContext();
			ctx.put("class", co);
			ctx.put("mapping", mo);
			if (pack.length() > 0) {
				ctx.put("package", pack);
			}
			ctx.put("mappername", mn);
			ctx.put("cparam", cp);
			CommonHelper ch = new CommonHelper();
			ch.setLogger(logger);
			ctx.put("tools", ch);
			ctx.put("header", GEN_TEMPLATE_DIR + "Header.vm");
			ctx.put("mappingTools", this);
			ctx.put("genInfos", genctxts.get(mo.getMapperName()));
			mmop.initContext(ctx);
			velocityEngine.getTemplate(GEN_TEMPLATE_DIR + "Mapping.vm").merge(ctx, fw);
			fw.flush();
			fw.close();
		} catch (Exception e) {
			e.printStackTrace();
			throw new PExceptionCompiler(e, "Problem while writing Mapping java file: " + fileName + ".java");
		}
	}

	// METHOD USED BY THE TEMPLATE //
	//-----------------------------//

	/**
	 * This method is a helper to reach the getInheritedClasses(Class, HashMap)
	 * method. Indeed in the velocity template is not possible to instanciate
	 * an object.
	 */
	public HashMap getInheritedClasses(Class c) {
		return getInheritedClasses(c, new HashMap());
	}

	/**
	 * This recursive method add in the HashMap @calculated all inherited class
	 * of the @current class parameter.
	 * @return an hashmap where keys are class names and their values are the
	 * direct inhertied class.
	 */
	private HashMap getInheritedClasses(Class current, HashMap calculated) {
		int parentNumber = current.getInheritedClassNumber();
		// Base case : No inherited classes
		if (parentNumber == 0)
			return calculated;

		String[] list = new String[parentNumber + 1];
		// Put the current class in first position into the list
		list[0] = current.getFQName();
		// Put the list into the structure
		calculated.put(current.getFQName(), list);
		//Full the list of inherited classes for the current class
		Iterator it = current.iterateInheritedClass();
		for (int i = 1; it.hasNext() && i < (parentNumber + 1); i++) {
			Class parent = (Class) it.next();
			list[i] = parent.getFQName();
			if (!calculated.containsKey(parent.getFQName()))
				getInheritedClasses(parent, calculated);
		}
		return calculated;
	}

	/**
	 * This generation helper method retrieves the string corresponding
	 * to the codding type (short value) associated to a PType.
	 */
	public String ptype2CTString(PType pt) throws PExceptionTyping {
		if (pt == null) {
			throw new PExceptionTyping("null parameter forbidden");
		}
		switch (pt.getTypeCode()) {
		case PType.TYPECODE_CHAR:
			return "PNamingContext.CTCHAR";
		case PType.TYPECODE_OBJCHAR:
			return "PNamingContext.CTOCHAR";
		case PType.TYPECODE_BYTE:
			return "PNamingContext.CTBYTE";
		case PType.TYPECODE_OBJBYTE:
			return "PNamingContext.CTOBYTE";
		case PType.TYPECODE_SHORT:
			return "PNamingContext.CTSHORT";
		case PType.TYPECODE_OBJSHORT:
			return "PNamingContext.CTOSHORT";
		case PType.TYPECODE_INT:
			return "PNamingContext.CTINT";
		case PType.TYPECODE_OBJINT:
			return "PNamingContext.CTOINT";
		case PType.TYPECODE_LONG:
			return "PNamingContext.CTLONG";
		case PType.TYPECODE_OBJLONG:
			return "PNamingContext.CTOLONG";
		case PType.TYPECODE_STRING:
			return "PNamingContext.CTSTRING";
		case PType.TYPECODE_DATE:
			return "PNamingContext.CTDATE";
		case PType.TYPECODE_CHARARRAY:
			return "PNamingContext.CTCHARARRAY";
		case PType.TYPECODE_BYTEARRAY:
			return "PNamingContext.CTBYTEARRAY";
		default:
			throw new PExceptionTyping("The following " + pt.getJormName()
									   + " type has'nt been allowed to be used as a PName field");
		}

	}

	/**
	 * This generation helper method retrieves the string corresponding
	 * to the instance of the PType.
	 */
	public String ptype2String(PType pt) throws PExceptionTyping {
		if (pt == null) {
			throw new PExceptionTyping("null parameter forbidden");
		}
		return pt.getProgName();
	}

	/**
	 * This method is a generation helper. It returns a string like the
	 * following regular expression: (".getGenClassMapping()")*
	 * where the number of repetition of the string is equals to the level
	 * parameter.
	 */
	public String getGenclass(int level) {
		String res = "";
		for (int i = 0; i < level; i++) {
			res += ".getGenClassMapping()";
		}
		return res;
	}

	/**
	 * This method calculates the number of indexes in a GenClass. This method
	 * should be moved into the meta object GenClass.
	 */
	public int getIndexNumber(GenClass gc) {
		Iterator it = gc.iterateIndexField();
		int res = 0;
		while (it.hasNext()) {
			res++;
			it.next();
		}
		return res;
	}

	public TypedElement getLastElement(GenClassRef gcr) {
		TypedElement o = gcr;
		while (o instanceof GenClassRef) {
			GenClassRef g = ((GenClassRef) o);
			if (g.isClassRef())
				o = g.getClassRef();
			else if (g.isGenClassRef())
				o = g.getClassRef();
			else if (g.isPrimitive())
				o = g.getPrimitiveElement();
		}
		return o;
	}

	public ArrayList getGenClassNames(GenClassRef gcr) {
		ArrayList gcns = new ArrayList(5);
		TypedElement o = gcr;
		while (o instanceof GenClassRef) {
			GenClassRef g = ((GenClassRef) o);
			gcns.add(g.getGenClassName());
			if (g.isClassRef())
				o = g.getClassRef();
			else if (g.isGenClassRef())
				o = g.getGenClassRef();
			else if (g.isPrimitive())
				o = g.getPrimitiveElement();
		}
		ArrayList res = new ArrayList(gcns.size());
		for (int i = 0; i < gcns.size(); i++) {
			String s = "new String[]{";
			for (int j = 0; j <= i; j++) {
				String elem = "\"" + gcns.get(j) + "\"";
				s += elem + (i == j ? "}" : ",");
			}
			res.add(s);
		}
		return res;
	}
}
