/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.adapter;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.api.PExceptionProtocol;

import java.util.Map;
import java.util.HashMap;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.sql.Statement;

/**
 * @author S.Chassande-Barrioz
 */
public class OracleAdapter extends BasicRdbAdapter {

	private final static String PROP_BASE = "org.objectweb.jorm.adapter.oracle.";
	private final static String ORACLESUBSTRING = "substr";
	private final static String ORACLEFIRSTLOCATE = "instr";

	public final static String[][] PROPS = {
		{PROP_BASE + "varchar", "256"},
		{PROP_BASE + "bytearray", "1999"},
		{PROP_BASE + "chararray", "1999"},
		{PROP_BASE + "varchar.pk", "256"}
	};

	public static final Map properties = new HashMap(PROPS.length);

	static {
		for (int i = 0; i < PROPS.length; i++) {
			properties.put(PROPS[i][0],
						   System.getProperty(PROPS[i][0], PROPS[i][1]));
		}
	}

	private final static int getDefaultSize(int idx) {
		return Integer.parseInt((String) properties.get(PROPS[idx][0]));
	}

	private int varcharSize = getDefaultSize(0);
	private int defaultBytearraySize = getDefaultSize(1);
	private int defaultChararraySize = getDefaultSize(2);
	private int varcharSizeInPk = getDefaultSize(3);

	public OracleAdapter() {
		super();
	}

	public void setVarcharSize(int s) {
		varcharSize = s;
	}

	public void setVarcharSizeInPk(int s) {
		varcharSizeInPk = s;
	}

	public boolean supportBatchPreparedStatement() {
		return false;
	}

	public String getSqlType(PType pt, boolean usedInPK, int size) throws PExceptionProtocol {
		if (pt == null) {
			throw new PExceptionProtocol("No Sql Type name associated to a null PType");
		}
		int realsize = size;
		switch (pt.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
		case PType.TYPECODE_OBJBOOLEAN:
		case PType.TYPECODE_BYTE:
		case PType.TYPECODE_OBJBYTE:
		case PType.TYPECODE_SHORT:
		case PType.TYPECODE_OBJSHORT:
		case PType.TYPECODE_LONG:
		case PType.TYPECODE_OBJLONG:
		case PType.TYPECODE_DOUBLE:
		case PType.TYPECODE_OBJDOUBLE:
		case PType.TYPECODE_BIGINTEGER:
		case PType.TYPECODE_BIGDECIMAL:
			return "NUMBER";
		case PType.TYPECODE_STRING:
			if (realsize == PType.NOSIZE) {
				realsize = (usedInPK ? varcharSizeInPk : varcharSize);
			}
			return "VARCHAR(" + realsize + ")";
		case PType.TYPECODE_BYTEARRAY:
			if (realsize == PType.NOSIZE) {
				realsize = defaultBytearraySize;
			}
			return (realsize > 2000 ? "LONG " : "") + "RAW(" + realsize + ")";
		case PType.TYPECODE_CHARARRAY:
			if (realsize == PType.NOSIZE) {
				realsize = defaultChararraySize;
			}
			return (realsize > 2000 ? "LONG " : "") + "RAW(" + realsize + ")";
		case PType.TYPECODE_SERIALIZED:
			return "BLOB";
		case PType.TYPECODE_DATE:
			return "DATE";
		default:
			return super.getSqlType(pt, usedInPK, size);
		}
	}

	public String getSubstringName() {
		return ORACLESUBSTRING;
	}

	public String getFirstLocateExpression(String substring, String instring) {
		return ORACLEFIRSTLOCATE + "(" + instring + ", " + substring + ")";
	}

	public String getIndexedLocateExpression(String instring, String substring,
											 String fromIndex) {
		return ORACLEFIRSTLOCATE + "(" + instring + ", " + substring
				+ ", " + fromIndex + ")";
	}

	public String getSqlGetterName(PType pt, String resultSet, int idx,
								   String sql)
			throws PExceptionProtocol {
		if (pt == null) {
			throw new PExceptionProtocol("Invalid parameter");
		}
		switch (pt.getTypeCode()) {
		case PType.TYPECODE_OBJBYTE:
			return "new Byte(" + resultSet + ".getByte(" + idx + "))";
		case PType.TYPECODE_OBJSHORT:
			return "new Short(" + resultSet + ".getShort(" + idx + "))";
		case PType.TYPECODE_OBJINT:
			return "new Integer(" + resultSet + ".getInt(" + idx + "))";
		default:
			return super.getSqlGetterName(pt, resultSet, idx, sql);
		}
	}

	public void writeTableAlias(String alias, StringBuffer sb) {
		sb.append(" ");
		sb.append(alias);
	}

	public String handleOrderBy(String query) {
		return "SELECT * FROM (" + query + ")";
	}

	public String getNextValInSequence(String seqName) {
		return "select " + seqName + " .nextval from dual";
	}

    public boolean existTable(Connection connection, String tableName) throws SQLException {
        boolean existtable = false;
        ResultSet rs = null;
        try {
            rs = connection.getMetaData().getTables(null, null, null, new String[]{"TABLE"});
            while (rs.next() && !existtable) {
                existtable = tableName.equalsIgnoreCase(rs.getString(3));
            }
        } finally {
            if (rs != null) {
                rs.close();
            }
        }
        return existtable;
    }

    public boolean existSequence(Connection connection, String seqName) throws SQLException {
        try {
            Statement s = connection.createStatement();
            s.execute(getCreateSequence(seqName));
            s.close();
        } catch (SQLException e) {
        }
        return true;
    }
}
