/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.generator;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.mapper.rdb.adapter.RdbAdapter;

import java.util.List;

/**
 * This class contains all required information related to a column to which
 * some field of a JORM class is mapped.
 * @author  P. Dechamboux
 */
public class RdbGenColumn {
	/**
	 * The name of the field that is mapped to this column.
	 */
	public String fieldName = null;
	/**
	 * Tells is this field is an hidden one.
	 */
	public boolean hiddenField = true;
	/**
	 * The name of the RDB column.
	 */
	public String columnName = null;
	/**
	 * The size of the RDB column in case of string, serialized,
	 * [byte|char]array...
	 */
	public int columnSize = PType.NOSIZE;
	/**
	 * The JORM type associated with this mapped field.
	 */
	public PType columnType = null;
	/**
	 * Tells if this column may store null values.
	 */
	public boolean columnNotNull = false;
	/**
	 * Tells if this column belongs to the table primary key.
	 */
	public boolean columnInPK = false;
	/**
	 * The table descrption which this column belongs to.
	 */
	public RdbGenTable columnTable = null;

	/**
	 * the sql type of the column. A null value means to use the value of the
	 * choosen by the adapter.
	 */
	public String columnSqlType = null;

	/**
	 * references a column in the main table in case of this column represents
	 * a join column in an external table. In other case this field is null.
	 */
	public RdbGenColumn joinCol = null;

	/**
	 * contains the association between a RdbGenJoin and a primitive element.
	 * This field is null is the current column is a column of the main table,
	 * because no join is needed of course.
	 */
	public List joins = null;
	public List pes = null;

	public RdbGenColumn getJoinCol() {
		return joinCol;
	}

	public String getColumnSqlType() {
		return columnSqlType;
	}

	public String getFieldName() {
		return fieldName;
	}

	public String getFieldName(RdbGenJoin rgj) throws PException {
		if (rgj != null && joins != null) {
			int idx = joins.indexOf(rgj);
			if (idx == -1) {
				throw new PException("No field found for the column "
									 + columnName + " with the join " + rgj);
			}
			return ((PrimitiveElement) pes.get(idx)).getName();
		}
		return fieldName;
	}

	public List getJoins() {
		return joins;
	}

	public boolean isHiddenField() {
		return hiddenField;
	}

	public String getColumnName() {
		return columnName;
	}

	public PType getColumnType() {
		return columnType;
	}

	public boolean isColumnNotNull() {
		return columnNotNull;
	}

	public boolean isColumnInPK() {
		return columnInPK;
	}

	public RdbGenTable getColumnTable() {
		return columnTable;
	}

	public String getColumnNotNull() {
		return "" + columnNotNull;
	}

	public String getColumnInPK() {
		return "" + columnInPK();
	}

	public boolean columnInPK() {
		if (columnTable.isMainTable) {
			return columnTable.genInfos.colInGenId(this);
		} else if (columnTable.colocatedTable) {
			return false;
		} else {
			return this.joinCol != null;
		}
	}

	/**
	 * Gets the SQL type associated with this column into the database for
	 * which we define the mapping.
	 * @return  The string containing the name of the relevant SQL type.
	 * @throws PException
	 */
	public String getSqlType(RdbAdapter adapter) throws PException {
		return (columnSqlType == null
				? adapter.getSqlType(columnType, columnInPK(), columnSize)
				: columnSqlType);
	}

	public String getFieldVal(String fieldval) {
		/*if (columnType == PTypeSpace.OBJCHAR) {
			return "new Character(((String) " + fieldval + ").charAt(0))";
		}*/
		return fieldval;
	}

	/**
	 * Constructs the string that defines the way to access to the relevant
	 * result part into a particular ResultSet.
	 * @param resultset The name of the ResultSet variable used to get the query
	 *                  result.
	 * @param pos       The position of the result to get.
	 * @return  The string defining the access to the relevant result part into
	 *          a particular ResultSet.
	 * @throws PException
	 */
	public String getSqlGet(String resultset, int pos, RdbAdapter adapter)
			throws PException {
		return adapter.getSqlGetterName(columnType, resultset, pos, columnSqlType);
	}

	/**
	 * Constructs the string that defines the way to assign to the relevant
	 * query parameter into a particular prepared statement.
	 * @param pstmt The name of the PreparedStatement variable used to set the
	 *              given query parameter.
	 * @param pos   The position of the query parameter to be assigned.
	 * @return  The string defining the assignement of the relevant query
	 *          parameter into a particular PreparedStatement.
	 * @throws PException
	 */
	public String getSqlSet(String pstmt, int pos, RdbAdapter adapter)
			throws PException {
		return adapter.getSqlSetterName(columnType, pstmt, pos, columnSqlType);
	}

	public int getSqlTypeCode(PType pt, RdbAdapter adapter) throws PException {
		return adapter.getSqlTypeCode(pt, columnSqlType);
	}

	public String getAccessorValue(RdbGenInfos genInfos,
								   boolean isSpecific) throws Exception {
		return getAccessorValue(genInfos, isSpecific, null);
	}

	public String getAccessorValue(RdbGenInfos genInfos,
								   boolean isSpecific, RdbGenJoin rgj) throws Exception {

		if (hiddenField) {
			if (genInfos.colInGenId(this)) {
				throw new PException("unmanaged case");
			} else {
				RdbGenRef gen_ref = genInfos.getGenRefOfColumn(this, rgj);
				if (isSpecific) {
					return "paccessor.paGet"
							+ RdbGenInfos.commonHelper.upperFL(gen_ref.fieldName)
							+ "(conn)";
				} else {
					return "paccessorGen.paGetRefField(pClassMapping."
							+ gen_ref.fieldName + "_NAME, conn)";
				}
			}
		} else {
			if (isSpecific) {
				return "paccessor.paGet"
						+ RdbGenInfos.commonHelper.upperFL(getFieldName(rgj))
						+ "()";
			} else {
				return "paccessorGen." +
						RdbGenInfos.commonHelper.getPaGetMethod(columnType)
						+ "(pClassMapping." + getFieldName(rgj) + "_NAME)";
			}
		}
	}

	public String getRefValue(RdbGenInfos genInfos,
							  boolean isSpecific) throws Exception {
		return getRefValue(genInfos, isSpecific, null);
	}

	public String getRefValue(RdbGenInfos genInfos,
							  boolean isSpecific, RdbGenJoin rgj) throws Exception {
		try {
			if (!hiddenField) {
				throw new PException("unmanaged case");
			}
			RdbGenRef gen_ref = null;
			if (genInfos.colInGenId(this)) {
				gen_ref = genInfos.genId;
				if (gen_ref.refColumn != null) {
					return "pname.encode" + columnType.getCodingName() + "()";
				} else {
					return "_pngId.pnGet" +
							RdbGenInfos.commonHelper.upperFL(gen_ref.getFieldName(this))
							+ "(null)";
				}
			} else {
				gen_ref = genInfos.getGenRefOfColumn(this, rgj);
				if (gen_ref == null) {
					return "JORM_GENERATION_ERROR";
				}
				if (gen_ref.refColumn != null) {
					return "pClassMapping."
							+ gen_ref.fieldName + "Pnc.encode" + columnType.getCodingName()
							+ "(" + getAccessorValue(genInfos, isSpecific, rgj) + ")";
				} else {
					return "png" + gen_ref.cnId + ".pnGet"
							+ RdbGenInfos.commonHelper.upperFL(
									gen_ref.getFieldName(this))
							+ "(null)";
				}

			}
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(-1);
			throw e;
		}
	}

	public String getColumnName(String tableSuffix) {
		return columnTable.tableName
				+ (tableSuffix == null ? "" : tableSuffix)
				+ "." + columnName;
	}

	public boolean isInJoin(RdbGenJoin rgj) {
		return (joins != null && joins.contains(rgj))
				|| rgj.getJoinColumnsInExt().contains(this);

	}
}
