/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.lib;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.lib.Mapper;
import org.objectweb.jorm.lib.JormConfigurator;
import org.objectweb.jorm.mapper.rdb.adapter.RdbAdapter;
import org.objectweb.jorm.mapper.rdb.adapter.RdbAdapterFactory;

import java.sql.Connection;
import java.sql.SQLException;
import javax.sql.DataSource;

/**
 * This mapper supports JDBC connection management.
 * @author	P. Dechamboux
 */
public class MapperJDBC extends Mapper implements PMapperRdb {

    private DataSource source = null;

    private RdbAdapter adapter;

	public MapperJDBC() throws PException {
		super();
	}

	public MapperJDBC(JormConfigurator jc) {
		super(jc);
	}

	/**
	 * Closes the given connection which has been allocated previously using
	 * getConnection.
	 * @param conn	The concerned JDBC connection.
	 */
	public void closeConnection(Object conn) throws PException {
		try {
			((Connection) conn).close();
		} catch (SQLException se) {
			throw new PException(se, "Problem while closing connection.");
		}
	}

	/**
	 * Allocates a connection for accessing the underlying relational DB.
	 */
	public Object getConnection() throws PException {
		if (source == null) {
			throw new PException("Cannot get an SQL connection: no datasource specified.");
		}
		try {
			return source.getConnection();
		} catch (SQLException se) {
			throw new PException(se, "Problem while allocating connection.");
		}
	}

	/**
	 * Allocates a connection for accessing the underlying relational DB.
	 * @param ctxt	The connection specification for the connection to be allocated.
	 */
	public Object getConnection(Object ctxt) throws PException {
		if (!(ctxt instanceof ConnectionSpecJDBC))
			throw new PException("Requires a JDBC connection specification.");
		ConnectionSpecJDBC cs = (ConnectionSpecJDBC) ctxt;
		try {
			if (cs.user != null && cs.passwd != null) {
				return source.getConnection(cs.user, cs.passwd);
			}
			return source.getConnection();
		} catch (SQLException se) {
			throw new PException(se,
								 "Problem while allocating connection with SPEC.");
		}
	}

	/**
	 * Assigns the information about the JDBC connection to be allocated
	 * through this mapper.
	 * @exception PExceptionProtocol	It is raised if a connection factory
	 * 									has already been assigned.
	 */
	public void setConnectionFactory(Object cf) throws PException {
		if (cf == null)
			return;
		if (cf instanceof DataSource) {
			source = (DataSource) cf;
			return;
		}
		throw new PException("Requires a JDBC connection specification.");
	}

	/**
	 * Returns the connection factory associated to this mapper.
	 */
	public Object getConnectionFactory() {
		return source;
	}

    public void start() throws PException {
        super.start();
        String mn = getMapperName();
        int idx = mn.indexOf('.');
        if (idx == -1) {
            mn = null;
        } else {
            mn = mn.substring(idx + 1);
        }
        adapter = RdbAdapterFactory.getTypeConverter(mn);
    }

    // IMPLEMENTATIOIN OF THE PMapperRdb INTERFACE //
    //---------------------------------------------//

    public RdbAdapter getRdbAdapter() {
        return adapter;
    }
}
