/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.lib;

import org.objectweb.jorm.api.PMapCluster;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.mapper.rdb.adapter.RdbAdapter;
import org.objectweb.transaction.api.ConnectionWrapper;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.util.Set;
import java.util.HashSet;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;

/**
 * Defines the PMapCluster used to manage relational schema for the RDB
 * mappers.
 * @author	P. Dechamboux
 */
public class RdbPMapCluster implements PMapCluster {

	/**
	 * The Jorm class names managed by this PMapCluster
	 */
	private Set jcNames;

	private Set unresolvedDependencies;
	/**
	 * The list of table managed by this PMapCluster. It contains Table instance.
	 */
	private ArrayList tables;

	private boolean structuresActive;
	private RdbPMappingStructuresManager manager;

	/**
	 * Narrows a connection requested to a mapper into an SQL connection.
	 * @param conn	A mapper connection.
	 * @return An SQL connection.
	 * @throws PException Thrown when narrowing is impossible.
	 */
	static Connection narrow2SQL(Object conn) throws PException {
		if (conn instanceof Connection)
			return (Connection) conn;
		if (conn instanceof ConnectionWrapper)
			return (Connection) ((ConnectionWrapper) conn).getNonCCIConnection();
		throw new PExceptionProtocol("Impossible to initialize the mapping structures with this connection type.");
	}

	/**
	 * Constructs a RdbPMapCluster.
	 * @param jcname	The first JORM class name to be assigned to this cluster.
	 * @param tname		The first table name to be assigned to this cluster.
	 * @param man		The manager associated to these mapping structures.
	 */
	RdbPMapCluster(String jcname, String tname, RdbPMappingStructuresManager man) {
		manager = man;
		jcNames = new HashSet(1);
		unresolvedDependencies = new HashSet(1);
		unresolvedDependencies.add(jcname);
		tables = new ArrayList(1);
		tables.add(new Table(tname,
                ((PMapperRdb) manager.getPMapper()).getRdbAdapter(), this));
		structuresActive = false;
	}

	public void classDefined(String jcname) {
		if (!jcNames.contains(jcname)) {
			jcNames.add(jcname);
			unresolvedDependencies.remove(jcname);
		}
	}

	/**
	 * Defines a new table column into this map cluster if it does not already
	 * exist. If it exist, it verifies that the redefinition is compatible with
	 * existing information.
	 * @param tname		The table name.
	 * @param cname		The column name.
	 * @param type		The SQL type of this column.
	 * @param notnull	The notnull status of this column.
	 * @throws PException Thrown if the column is redefined with incompatible
	 * information.
	 */
	void addTableColumn(String tname, String cname, String type,
						boolean notnull, boolean ispkcol, boolean isMaster) throws PException {
		Table curt = null;
		Iterator it = tables.iterator();
		while (it.hasNext()) {
			curt = (Table) it.next();
			if (curt.name.equals(tname)) {
				break;
			}
			curt = null;
		}
		if (curt == null) {
			// no such table defined yet: create a new one
			curt = new Table(tname,
                    ((PMapperRdb) manager.getPMapper()).getRdbAdapter(), this);
			tables.add(curt);
		}
		Column c = curt.getColumn(cname);
		if (c == null) {
			curt.addColumn(cname, type, notnull, ispkcol, isMaster);
		} else if (isMaster) {
			if (c.isMaster) {
				if (c.type != type) {
					if (ispkcol && !c.isPkCol) {
						c.notNull = notnull;
						c.isPkCol = ispkcol;
						c.type = type;
					} else if (!ispkcol && c.isPkCol) {
						//nothing to do
					} else {
						throw new PException(
								"Incompatible column definition: existing: " + c
								+ " asked : Column (name:" + cname
								+ ", type:" + type
								+ ", notNull:" + notnull
								+ ", isMaster:" + isMaster
								+ ", isPkCol:" + ispkcol
								+ ")");
					}
				}
			} else {
				c.isMaster = true;
				c.notNull = notnull;
				c.isPkCol = ispkcol;
				c.type = type;
			}
		}
	}

	/**
	 * Looks for this JORM class name into this map cluster.
	 * @param jcname	The JORM class name.
	 * @return true if this JORM class name is already assigned to this cluster.
	 */
	boolean containClass(String jcname) {
		return jcNames.contains(jcname);
	}

	/**
	 * Looks for this specified column into this map cluster.
	 * @param tname		The table name.
	 * @param cname		The column name.
	 * @param type		The SQL type of this column.
	 * @param notnull	The notnull status of this column.
	 * @throws PException Thrown if the column is redefined with incompatible
	 * information.
	 */
	boolean containColumn(String tname, String cname, String type,
						  boolean notnull) throws PException {
		Table curt = null;
		Iterator it = tables.iterator();
		while (it.hasNext()) {
			curt = (Table) it.next();
			if (curt.name.equals(tname)) {
				break;
			}
			curt = null;
		}
		if (curt == null) {
			// no such table defined yet : create a new one
			return false;
		}
		return curt.containColumn(cname, type, notnull);
	}

	/**
	 * Looks for this table name into this map cluster.
	 * @param tname	The table name.
	 * @return true if this table name is already assigned to this cluster.
	 */
	boolean containTable(String tname) {
		Iterator it = tables.iterator();
		while (it.hasNext()) {
			Table cur = (Table) it.next();
			if (cur.name.equals(tname))
				return true;
		}
		return false;
	}

	/**
	 * Looks for this table name into this map cluster.
	 * @param tname	The table name.
	 * @return true if this table name is already assigned to this cluster.
	 */
	Table getTable(String tname) {
		Iterator it = tables.iterator();
		while (it.hasNext()) {
			Table cur = (Table) it.next();
			if (cur.name.equals(tname))
				return cur;
		}
		return null;
	}

	/**
	 * Merges map cluster passed in parameter with this one. Clusters are merged
	 * either because they share a table name or a JORM class name.
	 * @param cl
	 */
	void merge(RdbPMapCluster cl) throws PException {
		jcNames.addAll(cl.jcNames);
		unresolvedDependencies.addAll(cl.getUnResolvedDependencies());
		unresolvedDependencies.removeAll(jcNames);
		for (int i = 0; i < cl.tables.size(); i++) {
			Table cur = (Table) cl.tables.get(i);
			Table t = getTable(cur.name);
			if (t == null) {
				tables.add(cur);
			} else {
				t.merge(cur);
			}
		}
	}

	// IMPLEMENTATION OF METHODS FROM THE PMapCluster INTERFACE

	/**
	 * Retrieves an Iterator over the collection of all JORM classes whose
	 * mapping structures have to be controlled altogether.
	 * @return  The Iterator over this collection.
	 */
	public Collection getClusterClasses() {
		if (unresolvedDependencies.isEmpty()) {
			return jcNames;
		} else {
			Set res = new HashSet(jcNames);
			res.addAll(unresolvedDependencies);
			return res;
		}
	}

	public boolean isDefined() {
		return unresolvedDependencies.isEmpty();
	}

	public Collection getUnResolvedDependencies() {
		return unresolvedDependencies;
	}

	public void addDependency(String jcname) {
		if (!jcNames.contains(jcname) && !unresolvedDependencies.contains(jcname)) {
			unresolvedDependencies.add(jcname);
		}
	}

	/**
	 * Creates the mapping structures defined by this map cluster.
	 * @param force	If true, the creation of an existing table leads to an exception.
	 * @throws PException   Thrown if it cannot be performed (especially
	 * if all or some of these mapping structures already exist.
	 */
	public void createMappingStructures(boolean force) throws PException {
		if (structuresActive) {
			throw new PException("Cannot change mapping structures while they are under use");
		}
		if (!isDefined()) {
			throw new PException("Cannot create hosting struture: the " +
								 "following dependencies has not been resolved: "
								 + unresolvedDependencies);
		}
		Object conn = manager.getConnection();
		try {
			Iterator it = tables.iterator();
			while (it.hasNext()) {
				Table cur = (Table) it.next();
				if (cur.existTable(RdbPMapCluster.narrow2SQL(conn))) {
					if (force) {
						throw new PException("Cannot create table " + cur.name + ": already exist");
					}
				} else {
					cur.createTable(RdbPMapCluster.narrow2SQL(conn));
				}
			}
		} finally {
			manager.closeConnection(conn);
		}
	}

	/**
	 * Deletes the data that have been inserted into the mapping structures
	 * defined by this map cluster.
	 * @throws PException   Thrown when the data store cannot perform this
	 * operation.
	 */
	public void deleteData() throws PException {
		if (!isDefined()) {
			throw new PException("Cannot remove data: the " +
								 "following dependencies has not been resolved: "
								 + unresolvedDependencies);
		}
		Object conn = manager.getConnection();
		try {
			Iterator it = tables.iterator();
			while (it.hasNext()) {
				Table cur = (Table) it.next();
				if (cur.existTable(RdbPMapCluster.narrow2SQL(conn))) {
					cur.deleteData(RdbPMapCluster.narrow2SQL(conn));
				} else {
					manager.logger.log(BasicLevel.WARN,
									   "Impossible to delete the data of the table '"
									   + cur.name + "': table does not exist.");
				}
			}
		} finally {
			manager.closeConnection(conn);
		}
	}

	/**
	 * Deletes all or part of the mapping structures defined by this map
	 * cluster when they exist. If they contain some data, they are also
	 * deleted.
	 * @throws PException   Thrown when the data store cannot perform this
	 * operation.
	 */
	public void deleteMappingStructures() throws PException {
		if (structuresActive) {
			throw new PException("Cannot change mapping structures while they are under use");
		}
		if (!isDefined()) {
			throw new PException("Cannot delete hosting struture: the " +
								 "following dependencies has not been resolved: "
								 + unresolvedDependencies);
		}
		Object conn = manager.getConnection();
		try {
			Iterator it = tables.iterator();
			while (it.hasNext()) {
				Table cur = (Table) it.next();
				if (cur.existTable(RdbPMapCluster.narrow2SQL(conn))) {
					cur.deleteTable(RdbPMapCluster.narrow2SQL(conn));
				}
			}
		} finally {
			manager.closeConnection(conn);
		}
	}

	/**
	 * Starts this map cluster. The following operation cannot be executed
	 * anymore: createMappingStructures, deleteMappingStructures, and
	 * updateMappingStructures.
	 * @throws PException
	 */
	public void start() throws PException {
		if (!structuresActive) {
			throw new PException("Cannot start map cluster: not yet inactive");
		}
		if (!isDefined()) {
			throw new PException("Cannot start map cluster: the following " +
								 "dependencies has not been resolved: "
								 + unresolvedDependencies);
		}
		structuresActive = true;
	}

	/**
	 * Stops this map cluster.
	 * @throws PException
	 */
	public void stop() throws PException {
		if (structuresActive) {
			throw new PException("Cannot stop map cluster: not yet active");
		}
		structuresActive = false;
	}

	/**
	 * Aligns existing mapping structures to the ones define by this map
	 * cluster. This operation allows schema evolution on existing data store.
	 * It is thus usually difficult to support in the general case.
	 * @throws PException   Thrown when the data store cannot perform this
	 * operation.
	 * @throws UnsupportedOperationException    Thrown if this operation is not
	 * supported by the mapping.
	 */
	public void updateMappingStructures()
			throws PException, UnsupportedOperationException {
		if (structuresActive) {
			throw new PException("Cannot change mapping structures while they are under use");
		}
		throw new UnsupportedOperationException();
	}
}

/**
 * Describes a table into a map cluster.
 */
class Table {
	String name;
	ArrayList columns;
    RdbAdapter adapter;
	RdbPMapCluster cluster;


	Table(String tname, RdbAdapter adapter, RdbPMapCluster cluster) {
		name = tname;
        this.adapter = adapter;
		this.cluster = cluster;
		columns = new ArrayList();
	}

	void addColumn(String cname, String type, boolean notnull, boolean ispkcol, boolean ismaster)
			throws PException {
		Column cur;
		cur = new Column();
		cur.name = cname;
		cur.type = type;
		cur.notNull = notnull;
		cur.isPkCol = ispkcol;
		cur.isMaster = ismaster;
		columns.add(cur);
	}

	void merge(Table t) throws PException {
		for (int i = 0; i < t.columns.size(); i++) {
			Column cur = (Column) t.columns.get(i);
			Column c = getColumn(cur.name);
			if (c == null) {
				columns.add(cur);
			} else {
				c.merge(cur);
			}
		}
	}

	boolean containColumn(String cname, String type,
						  boolean notnull) throws PException {
		Column cur;
		Iterator it = columns.iterator();
		while (it.hasNext()) {
			cur = (Column) it.next();
			if (cur.name.equals(cname)) {
				return true;
			}
		}
		return false;
	}

	Column getColumn(String cname) {
		Column cur;
		Iterator it = columns.iterator();
		while (it.hasNext()) {
			cur = (Column) it.next();
			if (cur.name.equals(cname)) {
				return cur;
			}
		}
		return null;
	}

	void createTable(Connection connection) throws PException {
		Statement stmt = null;
		StringBuffer sb = new StringBuffer("CREATE TABLE " + name + " (");
		try {
			stmt = connection.createStatement();
			String sep = "";
			Iterator it = columns.iterator();
			while (it.hasNext()) {
				sb.append(sep);
				sep = ", ";
				Column col = (Column) it.next();
				sb.append(col.name);
				sb.append(" ");
				sb.append(col.type);
				if (col.notNull && col.type.indexOf("NOT NULL") == -1) {
					sb.append(" NOT NULL");
				}
			}
			sep = "";
			it = columns.iterator();
			while (it.hasNext()) {
				Column col = (Column) it.next();
				if (!col.isPkCol) {
					continue;
				}
				if (sep.equals("")) {
					sb.append(", PRIMARY KEY (");
				}
				sb.append(sep);
				sep = ", ";
				sb.append(col.name);
			}
			if (!sep.equals("")) {
				sb.append(")");
			}
			sb.append(")");
			stmt.execute(sb.toString());
		} catch (java.sql.SQLException se) {
			throw new PExceptionIO(se, "Impossible to create the table '"
					+ name + "' of the cluster " + cluster.getClusterClasses()
					+ " : " + sb.toString());
		} finally {
			if (stmt != null) {
				try {
					stmt.close();
				} catch (SQLException e) {
				}
			}
		}
	}

	void deleteData(Connection connection) throws PException {
		Statement stmt = null;
		try {
			stmt = connection.createStatement();
			stmt.execute("DELETE FROM " + name);
		} catch (java.sql.SQLException se) {
			throw new PExceptionIO(se);
		} finally {
			if (stmt != null) {
				try {
					stmt.close();
				} catch (SQLException e) {
				}
			}
		}
	}

	void deleteTable(Connection connection) throws PException {
		Statement stmt = null;
		try {
			stmt = connection.createStatement();
			stmt.execute("DROP TABLE " + name);
		} catch (java.sql.SQLException se) {
			throw new PExceptionIO(se);
		} finally {
			if (stmt != null) {
				try {
					stmt.close();
				} catch (SQLException e) {
				}
			}
		}
	}

	boolean existTable(Connection connection) throws PException {
		try {
			return adapter.existTable(connection, name);
		} catch (SQLException e) {
			throw new PException(e, "Pb while testing the existence of table " + name);
		}
	}
}

/**
 * Describes a column into a table description into a map cluster.
 */
class Column {
	String name;
	String type;
	boolean notNull;
	boolean isPkCol;
	boolean isMaster;

	void merge(Column c) throws PException {
		if (type != c.type) {
			throw new PException("Impossible to merge the column " + name
								 + ": column type does not match");
		}
		if (c.isMaster) {
			notNull = c.notNull;
			isPkCol |= c.isPkCol;
		}
	}

	public String toString() {
		return "Column (name:" + name
				+ ", type:" + type
				+ ", notNull:" + notNull
				+ ", isMaster:" + isMaster
				+ ", isPkCol:" + isPkCol
				+ ")";
	}
}
