/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.mi2xml;

import org.objectweb.jorm.metainfo.api.*;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.lib.Predicate;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.mi2xml.lib.BasicMappingDomtreeBuilder;
import org.objectweb.jorm.mapper.rdb.metainfo.*;
import org.objectweb.util.monolog.api.BasicLevel;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.util.*;

/**
 * BasicRdbMappingDomTreeGenerator extends the BasicMappingDomTreeGenerator class.
 *
 * BasicRdbMappingDomTreeGenerator generates an org.w3c.dom.Document object for
 * a ClassMapping object or a GenClassMapping object.
 */
public class RdbDomtreeBuilder extends BasicMappingDomtreeBuilder {
	// TODO: create a separate class
	public final static String RDB_CLASS_MAPPING = "rdb-class-mapping";
	public final static String COLOCATED_CLASS = "colocated-class";
	public final static String RDB_TABLE_SPEC = "rdb-table-spec";
	public final static String ID_MAPPING = "id-mapping";
	public final static String RDB_CLASS_REF_MAPPING = "rdb-class-ref-mapping";
	public final static String RDB_GEN_CLASS_REF_MAPPING = "rdb-gen-class-ref-mapping";
	public final static String RDB_REF_MAPPING = "rdb-ref-mapping";
	public final static String RDB_EXTERNAL_TABLE_SPEC = "rdb-external-table-spec";
	public final static String RDB_JOIN = "rdb-join";
	public final static String RDB_FILTER_SPEC = "rdb-filter-spec";
	public final static String RDB_COLUMN_REF = "rdb-column-ref";
	public final static String RDB_COLUMN_SPEC = "rdb-column-spec";
	public final static String RDB_COLUMN_MATCH = "rdb-column-match";
	public final static String RDB_GEN_CLASS_MAPPING = "rdb-gen-class-mapping";
	public final static String RDB_EXTENSION_REF_MAPPING = "rdb-extension_ref_mapping";

	public final static String LINKEND_SEP = "/";

	/**
	 * Builds a new rdb domtree generator.
	 * This class is loaded by a classloader.
	 */
	public RdbDomtreeBuilder() {
		// keep empty
	}

	/**
	 * Adds a rdb-class-mapping element to a mapping element.
	 * @param   document        an org.w3c.dom.Document object,
	 *          mappingElement  a mapping element,
	 *          classMapping    a <tt>ClassMapping</tt> object.
	 */
	public void processClassMapping(Document document, Element mappingElement,
									ClassMapping classMapping) throws PException {
		Element cmElement = document.createElement(RDB_CLASS_MAPPING);
		//cmElement.setAttribute("link-end", currentClass.getName());
		mappingElement.appendChild(cmElement);
		// Processes the dependencies among classes.
		if (!(classMapping.getDependencies().isEmpty())) {
			Iterator classNameIterator = classMapping.getDependencies().iterator();
			while (classNameIterator.hasNext()) {
				String jormClassName = (String) classNameIterator.next();
				Element ccElement = document.createElement(COLOCATED_CLASS);
				ccElement.setAttribute("class-name", jormClassName);
				if (logger.isLoggable(BasicLevel.DEBUG)) {
					logger.log(BasicLevel.DEBUG,
							   "begin =<" + ccElement.getTagName() + ">");
					logger.log(BasicLevel.DEBUG,
							   " colocated class =<" + jormClassName + ">");
				}
				cmElement.appendChild(ccElement);
				if (logger.isLoggable(BasicLevel.DEBUG)) {
					logger.log(BasicLevel.DEBUG,
							   "end =<" + ccElement.getTagName() + ">");
				}
			}
		}
		// Processes the RdbTable object.
		RdbTable table = ((RdbClassMultiMapping) classMapping).getRdbTable();
		String colocated = (table.isColocated() ? "TRUE" : "FALSE");
		String colocatedMaster = (table.isColocatedMaster() ? "TRUE" : "FALSE");
		Element tableElement = document.createElement(RDB_TABLE_SPEC);
		tableElement.setAttribute("table-name", table.getName());
		tableElement.setAttribute("colocated", colocated);
		tableElement.setAttribute("colocated-master", colocatedMaster);
		cmElement.appendChild(tableElement);
		processRdbTableSpec(document, tableElement, table);
		// Processes the RdbExternalTable objects.
		if (!((RdbClassMultiMapping) classMapping).getRdbExternalTables().isEmpty()) {
			Iterator etIterator = ((RdbClassMultiMapping) classMapping).getRdbExternalTables().iterator();
			while (etIterator.hasNext()) {
				RdbExternalTable externalTable = (RdbExternalTable) etIterator.next();
				String etcolocated = (externalTable.isColocated() ? "TRUE" : "FALSE");
				String etcolocatedMaster = (externalTable.isColocatedMaster() ? "TRUE" : "FALSE");
				Element etElement = document.createElement(RDB_EXTERNAL_TABLE_SPEC);
				if (logger.isLoggable(BasicLevel.DEBUG)) {
					logger.log(BasicLevel.DEBUG,
							   "begin =<" + etElement.getTagName() + ">");
				}
				etElement.setAttribute("table-name", externalTable.getName());
				etElement.setAttribute("colocated", etcolocated);
				etElement.setAttribute("colocated-master", etcolocatedMaster);
				//etElement.setAttribute("read-only", readOnly);
				cmElement.appendChild(etElement);
				processRdbExternalTableSpec(document, etElement, externalTable);
				if (logger.isLoggable(BasicLevel.DEBUG)) {
					logger.log(BasicLevel.DEBUG,
							   "end =<" + etElement.getTagName() + ">");
				}
			}
		}
		// Processes the RdbFilter object.
		RdbFilter filter = ((RdbClassMultiMapping) classMapping).getRdbFilter();
		if (filter != null) {
            Element filterElement = document.createElement(RDB_FILTER_SPEC);
            if (logger.isLoggable(BasicLevel.DEBUG)) {
		        logger.log(BasicLevel.DEBUG,
                           "begin =<" + filterElement.getTagName() + ">");
			}
            cmElement.appendChild(filterElement);
            Set filterColumns = filter.getFilterColumnNames();
            if (! filterColumns.isEmpty()) {
                Iterator fcIterator = filterColumns.iterator();
			    while (fcIterator.hasNext()) {
				    String colName = (String) fcIterator.next();
                    Element colRefElement = document.createElement(RDB_COLUMN_REF);
                    colRefElement.setAttribute("column-name", colName);
                    filterElement.appendChild(colRefElement);
                    if (logger.isLoggable(BasicLevel.DEBUG)) {
		                logger.log(BasicLevel.DEBUG,
                           "column-name <" + colName + ">");
			        }
                    List predicates = filter.getPredicates(colName);
                    if (! predicates.isEmpty()) {
                        Iterator pIterator = predicates.iterator();
			            while (pIterator.hasNext()) {
                            Predicate predicate = (Predicate) pIterator.next();
                            Element predicateElem = document.createElement(predicate.getOperator());
                            predicateElem.setAttribute("value", predicate.getValue());
                            colRefElement.appendChild(predicateElem);
                        }
                    }
                }
            }
            if (logger.isLoggable(BasicLevel.DEBUG)) {
		        logger.log(BasicLevel.DEBUG,
                           "end =<" + filterElement.getTagName() + ">");
			}
        }
		// Processes the IdentifierMapping object.
		IdentifierMapping identifierMapping = classMapping.getIdentifierMapping();
        if ( identifierMapping != null) {
		    // Gets the referenced NameDef object
		    NameDef idNameDef = (NameDef) identifierMapping.getLinkedMO();
		    // Gets the parent of the NameDef object
		    Class parent = (Class) idNameDef.getParent();
		    String linkend = null;
		    //TODO: simplify code - see JormPathHelper
		    if (parent == currentClass) {
			    linkend = idNameDef.getName();
		    } else {
			    linkend = LINKEND_SEP + parent.getFQName() + LINKEND_SEP + idNameDef.getName();
		    }
		    Element idNameDefElement = document.createElement(ID_MAPPING);
		    //idNameDefElement.setAttribute("link-end", (String) metaobject2idvalue.get(idNameDef));
		    idNameDefElement.setAttribute("link-end", linkend);
		    cmElement.appendChild(idNameDefElement);
        }

		// Processes the ReferenceMapping objects.
		if (!(classMapping.getReferenceMappings().isEmpty())) {
			Iterator rmIterator = classMapping.getReferenceMappings().iterator();
			while (rmIterator.hasNext()) {
				ReferenceMapping referenceMapping = (ReferenceMapping) rmIterator.next();
				NameDef refNameDef = (NameDef) referenceMapping.getLinkedMO();
                String ruleName = referenceMapping.getRuleName();
				// Gets the parent of the Reference object that contains this NameDef object.
				Reference reference = (Reference) refNameDef.getParent();
                String linkend = null;
                if (reference instanceof Extension) {
                    Extension extension = (Extension) reference;
                    linkend = ( refNameDef.getName().equals("")
                        ? LINKEND_SEP + extension.getFQName()
                        : LINKEND_SEP + extension.getFQName() + LINKEND_SEP + refNameDef.getName());
                } else {
				    Class clazz = (Class) reference.getParent();
				    if (clazz == currentClass) {
                        linkend = ( refNameDef.getName().equals("")
                            ? reference.getName()
                            : reference.getName() + LINKEND_SEP + refNameDef.getName());
				    } else {
					    linkend = LINKEND_SEP + clazz.getFQName() + LINKEND_SEP + reference.getName() + LINKEND_SEP + refNameDef.getName();
				    }
                }
				Element refElement = null;
				if (refNameDef.getParent() instanceof ClassRef) {
					refElement = document.createElement(RDB_CLASS_REF_MAPPING);
				} else if (refNameDef.getParent() instanceof GenClassRef) {
					refElement = document.createElement(RDB_GEN_CLASS_REF_MAPPING);
                } else if (refNameDef.getParent() instanceof Extension) {
					refElement = document.createElement(RDB_EXTENSION_REF_MAPPING);
				}
				//refElement.setAttribute("link-end", (String) metaobject2idvalue.get(refNameDef));
				refElement.setAttribute("link-end", linkend);
				refElement.setAttribute("rule-name", ruleName);
				cmElement.appendChild(refElement);
			}
		}
	}

	/**
	 * Adds rdb-column-spec elements to a rdb-table-spec element.
	 * @param   document        an org.w3c.dom.Document object,
	 *          tableElement    a rdb-table-spec element,
	 *          table           an <tt>RdbTable</tt> object.
	 */
	private void processRdbTableSpec(Document document, Element tableElement, RdbTable table) {
		// Processes the PrimitiveElementMapping objects.
		if (!(table.getPrimitiveElementMappings().isEmpty())) {
			Iterator pemIterator = table.getPrimitiveElementMappings().iterator();
			while (pemIterator.hasNext()) {
				RdbPrimitiveElementMapping pem = (RdbPrimitiveElementMapping) pemIterator.next();
				String notNull = (pem.isNotNull() ? "TRUE" : "FALSE");
				PrimitiveElement pe = (PrimitiveElement) pem.getLinkedMO();
				Element csElement = document.createElement(RDB_COLUMN_SPEC);
				//csElement.setAttribute("link-end", (String) metaobject2idvalue.get(pe));
				// if pe is a field or a scalarfield, link-end is set with the corresponding field name.
				// if pe defines the primitive type of a generic class, link-end is set with the
				// corresponding gen-class name.
                MetaObject parent = pe.getParent();
                String linkend = pe.getName();
                if (parent instanceof Class) {
                    boolean isInherited = (parent != currentClass);
                    if (isInherited) {
                        linkend = LINKEND_SEP + ((Class) parent).getFQName() + LINKEND_SEP + pe.getName();
                    }
                }
			    logger.log(BasicLevel.DEBUG,
							   " rdb-column-spec link-end =<" + linkend + ">");
				csElement.setAttribute("link-end", linkend);
				csElement.setAttribute("column-name", pem.getName());
				csElement.setAttribute("sql-type", pem.getType());
				csElement.setAttribute("not-null", notNull);
				csElement.setAttribute("join-name", "");
				tableElement.appendChild(csElement);
			}
		}
	}

	/**
	 * Adds rdb-join and rdb-column-spec elements to a rdb-external-table-spec element.
	 * @param   document        an org.w3c.dom.Document object,
	 *          etElement       a rdb-external-table-spec element,
	 *          externalTable   a <tt>RdbExternalTable</tt> object.
	 */
	private void processRdbExternalTableSpec(Document document, Element etElement,
											 RdbExternalTable externalTable) throws PException {
		// Processes the Join objects.
		Iterator joinIterator = externalTable.getRdbJoins().iterator();
		while (joinIterator.hasNext()) {
			RdbJoin join = (RdbJoin) joinIterator.next();
			Element joinElement = document.createElement(RDB_JOIN);
			joinElement.setAttribute("name", join.getName());
			etElement.appendChild(joinElement);
			processRdbJoin(document, joinElement, join);
		}
		// Processes the PrimitiveElement objects.
		if (!(externalTable.getPrimitiveElementMappings().isEmpty())) {
			Iterator peIterator = externalTable.getPrimitiveElementMappings().iterator();
			Element csElement = null;
			while (peIterator.hasNext()) {
				RdbPrimitiveElementMapping pem = (RdbPrimitiveElementMapping) peIterator.next();
				String notNull = (pem.isNotNull() ? "TRUE" : "FALSE");
				PrimitiveElement pe = null;
				RdbJoin join = null;
				Map join2pe = pem.getPrimitiveElementByRdbJoin();
				if (join2pe != null) {
					// the column is associated with several PrimitiveElement objects
					joinIterator = join2pe.keySet().iterator();
					while (joinIterator.hasNext()) {
						join = (RdbJoin) joinIterator.next();
						pe = (PrimitiveElement) join2pe.get(join);
						csElement = document.createElement(RDB_COLUMN_SPEC);
						csElement.setAttribute("link-end", pe.getName());
						csElement.setAttribute("column-name", pem.getName());
						csElement.setAttribute("sql-type", pem.getType());
						csElement.setAttribute("not-null", notNull);
						csElement.setAttribute("join-name", join.getName());
						etElement.appendChild(csElement);
					}
				} else {
					pe = (PrimitiveElement) pem.getLinkedMO();
					join = pem.getJoinByPrimitiveElement(pe);
					if (join == null) {
						throw new PException("No join defined in the external table for the field: "
											 + pe.getName());
					}
					csElement = document.createElement(RDB_COLUMN_SPEC);
					csElement.setAttribute("link-end", pe.getName());
					csElement.setAttribute("column-name", pem.getName());
					csElement.setAttribute("sql-type", pem.getType());
					csElement.setAttribute("not-null", notNull);
					csElement.setAttribute("join-name", join.getName());
					etElement.appendChild(csElement);
				}
				if (logger.isLoggable(BasicLevel.DEBUG)) {
					logger.log(BasicLevel.DEBUG,
							   "begin =<" + csElement.getTagName() + ">");
				}
				//csElement.setAttribute("link-end", (String) metaobject2idvalue.get(pe));
				if (logger.isLoggable(BasicLevel.DEBUG)) {
					logger.log(BasicLevel.DEBUG,
							   "end =<" + csElement.getTagName() + ">");
				}
			}
		}
	}

	/**
	 * Adds rdb-column-match elements to a rdb-join element.
	 * @param   document     an org.w3c.dom.Document object,
	 *          joinElement  a rdb-join element,
	 *          RdbJoin      a <tt>RdbJoin</tt> object.
	 */
	private void processRdbJoin(Document document, Element joinElement, RdbJoin join) {
		// Processes the join columns.
		Iterator column1Iterator = join.getPTJoinColumnNames().iterator();
		Iterator column2Iterator = join.getETJoinColumnNames().iterator();
		while (column1Iterator.hasNext()) {
			String column1 = (String) column1Iterator.next();
			String column2 = (String) column2Iterator.next();
			Element cmElement = document.createElement(RDB_COLUMN_MATCH);
			cmElement.setAttribute("column1", column1);
			cmElement.setAttribute("column2", column2);
			;
			joinElement.appendChild(cmElement);
		}
	}

	/**
	 * Adds a rdb-gen-class-mapping element to a mapping element.
	 * @param   document            an org.w3c.dom.Document object,
	 *          mappingElement      a mapping element,
	 *          genClassMapping     a <tt>GenClassMapping</tt> object.
	 */
	public void processGenClassMapping(Document document, Element mappingElement,
									   GenClassMapping genClassMapping) {
		GenClassRef genClassRef = (GenClassRef) genClassMapping.getLinkedMO();
		Element gcmElement = document.createElement(RDB_GEN_CLASS_MAPPING);
		gcmElement.setAttribute("link-end", (String) metaobject2idvalue.get(genClassRef));
		mappingElement.appendChild(gcmElement);
		// Processes the RdbTable object.
		RdbTable table = ((RdbGenClassMapping) genClassMapping).getRdbTable();
		String colocated = (table.isColocated() ? "TRUE" : "FALSE");
		String colocatedMaster = (table.isColocatedMaster() ? "TRUE" : "FALSE");
		Element tableElement = document.createElement(RDB_TABLE_SPEC);
		tableElement.setAttribute("table-name", table.getName());
		tableElement.setAttribute("colocated", colocated);
		tableElement.setAttribute("colocated-master", colocatedMaster);
		gcmElement.appendChild(tableElement);
		processRdbTableSpec(document, tableElement, table);
		// Processes the IdentifierMapping object.
		IdentifierMapping identifierMapping = genClassMapping.getIdentifierMapping();
		NameDef idNameDef = (NameDef) identifierMapping.getLinkedMO();
		Element idNameDefElement = document.createElement(ID_MAPPING);
		//idNameDefElement.setAttribute("link-end", (String) metaobject2idvalue.get(idNameDef));
		idNameDefElement.setAttribute("link-end", idNameDef.getName());
		gcmElement.appendChild(idNameDefElement);
		// Processes the ReferenceMapping object.
		if (genClassMapping.getReferenceMapping() != null) {
			ReferenceMapping referenceMapping = genClassMapping.getReferenceMapping();
			NameDef refNameDef = (NameDef) referenceMapping.getLinkedMO();
			Element refElement = document.createElement(RDB_REF_MAPPING);
			//refElement.setAttribute("link-end", (String) metaobject2idvalue.get(refNameDef));
			refElement.setAttribute("link-end", refNameDef.getName());
			refElement.setAttribute("rule-name", referenceMapping.getRuleName());
			gcmElement.appendChild(refElement);
		}
	}
}

