/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.metainfo.api;

import org.objectweb.jorm.type.api.PType;

import java.util.Iterator;
import java.util.Map;
import java.util.Collection;

/**
 * A Class interface defines methods to describe a Class meta object.
 * A Class is composed by a set of fields (primitive, reference class or
 * reference generic class), a set of inherited classes, a mapping class,
 * can be abstract or not and has a name.
 * @author	X. Spengler
 */
public interface Class extends MetaObject {
	/**
	 * Allows to retrieve the file name of this class MO.
	 * @return  The file name.
	 */
	String getFileName();

	/**
	 * Allows to assign the file name of this class MO.
	 * @param filename  The file name.
	 */
	void setFileName(String filename);

	/**
	 * Allows to know if the current class is an abstract class or not.
	 *
	 * @return	true, if the current class is an abstract class, and false,
	 *          if the current class is not abstract
	 */
	boolean isAbstract();

	void setAbstract(boolean isAbstract);

	/**
	 * Allows to know the name of the current class object.
	 * @return	the string representation of the class name
	 */
	String getName();

	/**
	 * Gets the fully qualified name (including the package name) associated
	 * with this class.
	 * @return	the string representation of the fully qualified class name
	 */
	String getFQName();

	/**
	 * Allows to obtain an existing Field.
	 * A TypedElement is a class field, and this method finds a field with its
	 * name (fieldName). This method is a generic method to return a
	 * TypedElement object.
	 *
	 * @param fieldName	the name of the class field
	 * @return	a field object. If the field does not exist, null is returned.
	 */
	TypedElement getTypedElement(String fieldName);

	/**
	 * Removes a typed element.
	 * @param fieldName is the name of the field to be removed
	 * @return the removed element or null if the field has not been found.
	 */
	TypedElement removeTypedElement(String fieldName);

	/**
	 * Creates a new NameDef object for the current Class object.
	 * Here we can define if the namedef is defined outside jorm or not.
	 * In the case where the namedef is already defined for the current class,
	 * the old one is used and returned.
	 * associated.
	 * @return	a new object used to describe the PName projection for
	 *          the current Class
	 */
	NameDef createNameDef();

	/**
	 * Creates an hiddenfield for the current Class object.
	 * An hidden field is used to map the pname of the class or the pname of a ref field.
	 * An hidden field is a scalarfield.
	 *
	 * @param fieldname     the name of the hiddenfield
	 * @param type	        the ptype of the hiddenfield
	 * @return	a new scalarfield object used to describe the hiddenfield
	 */

	ScalarField createHiddenField(String fieldname, PType type, int size);

	/**
	 * Provides an iterator over the all the hiddenfield of the class including inherited fields.
	 * @return the iterator embeding scalarfield object describing the hiddenfields.
	 */
	Iterator iterateAllHiddenField();

	/**
	 * Provides an iterator over the all the hiddenfield of the class excluding inherited fields.
	 * @return the iterator embeding scalarfield object describing the hiddenfields.
	 */
	Iterator iterateHiddenField();

	/**
	 * Retrieve the hiddenfield identified by its name in the current class.
	 * @param fieldname the name of the hiddenfield.
	 * @return the scalarfield object describing the hiddenfield or null if the field does not exist in the class.
	 */
	ScalarField getHiddenField(String fieldname);


	/**
	 * Returns an iterator on existing NameDef for the current Class.
	 * This method returns an empty iterator if no namedef exists in this Class.
	 *
	 * @return	an iterator on NameDef, or an empty iterator if no namedef is
	 *          defined
	 */
	Iterator iterateNameDef();

	/**
	 * Returns a NameDef object from a given mapperName and mappingName.
	 * If the namedef does not exist, null is returned.
	 *
	 * @param projectName   the name of the class project
	 * @return  an existing namedef object if exists, either null
	 */
	NameDef getNameDef(String projectName);

	/**
	 * Allows to obtain a new PrimitiveElement from a class field name an its
	 * type.
	 * A PrimitiveElement is a Class field. If the class field already exists,
	 * it is returned.
	 *
	 * @param fieldName	the name of the primitive field
	 * @param type      the PType of the primitive field
	 * @return	a new PrimitiveElement object, or the existing one if already
	 *          defined
	 */
	PrimitiveElement createPrimitiveElement(String fieldName, PType type, int size);

	/**
	 * Allows to obtain a new ClassRef from a field name an its class.
	 * A ClassRef object is a field which is represented by a reference to a
	 * Class. If the field already exists, it is returned.
	 *
	 * @param fieldName	the name of the field
	 * @param class_	the class meta-object which represents the reference of
	 * 					the field
	 * @return	a new ClassRef object, or the existing one if already defined
	 */
	ClassRef createClassRef(String fieldName, Class class_);

	/**
	 * Allows to obtain a new GenClassRef from a field name an its generic
	 * class.
	 * A GenClassRef object is a field which is represented by a
	 * reference to a generic class. If the field already exists,
	 * it is returned.
	 *
	 * @param fieldName	the name of the field
	 * @param genName	the GenClass object which represents the reference of
	 *                  the field
	 * @return	a new GenClassRef object, or the existing one if already
	 *          defined
	 */
	GenClassRef createGenClassRef(String fieldName, String genName);

	/**
	 * Returns an iterator on existing class fields.
	 * If no field is defined, an empty iterator is returned.
	 * This iterator contains PrimitiveElements, ClassRef and GenClassRef
	 * objects. The top level object is TypedElement.
	 *
	 * @return	an iterator on fields, or null if there is no defined field
	 */
	Iterator iterateField();

	/**
	 * Returns an iterator on existing class fields + fields from super classes.
	 * This iterator contains PrimitiveElement, ClassRef and GenClassRef
	 * objects.
	 *
	 * @return	an iterator on all defined fields, plus fields from super
	 */
	Iterator iterateAllField();

	/**
	 * Allows us to obtain all the inherited classes from the current class object.
	 * This iterator contains Class objects. In the case where there is no
	 * inherited class, an empty iterator is returned.
	 *
	 * @return	an iterator on inherited classes (Class objets). If no object
	 *          exists, an empty iterator is returned.
	 */
	Iterator iterateInheritedClass();

	/**
	 * Allows us to obtain all the Extension objects from the current class object.
     * If there is no Extension object, an empty iterator is returned.
	 *
	 * @return	an iterator on Extension classes. If no object
	 *          exists, an empty iterator is returned.
	 */
	public Iterator iterateExtension();

	/**
	 * Returns a collection of Extension objects.
	 * @return	 a collection.
	 */
	public Collection getExtensions();

	/**
	 * Returns an Extension object.
     *
     * @param    fqcn   a full qualified class name.
	 * @return	 an Extension object.
	 */
	public Extension getExtension(String fqcn);

	/**
	 * Adds a new inherited class to the list of the inherited classes for the
	 * current class object.
	 * If the class already exists, nothing is done.
	 *
	 * @param class_	the inherited class to add to inherited classes list
     *
     * @return  an Extension object.
	 */
	Extension addInheritedClass(Class class_);

	/**
	 * Returns the number of inherited classes for the current Class object.
	 *
	 * @return  the number of inherited classes
	 */
	int getInheritedClassNumber();

	/**
	 * Constructs an Hashtable with inherited class
	 * @param table the hashtable to build
	 */
	void addAllInheritedClass(Map table);

	/**
	 * Returns the number of fields which are references to classes.
	 *
	 * @return  the number of fieldRef
	 */
	int getFieldRefNumber();

	/**
	 * Specifies that this class is inherited by another one within the
	 * set of compiled ones.
	 */
	void setInherited();

	/**
	 * Tests if this class is inherited by another one.
	 */
	boolean isInherited();

	/**
	 * Returns a ClassProject object.
	 *
	 * @param  projectName       a project name.
	 * @return a ClassProject object.
	 */
	ClassProject getClassProject(String projectName);

	/**
	 * Returns a collection of ClassProject objects.
	 * @return	 a collection.
	 */
	Collection getClassProjects();

	/**
	 * Adds a ClassProject object.
	 *
	 * @param  projectName       a project name,
	 *         classProject	    a ClassProject object.
	 */
	void addClassProject(String projectName, ClassProject classProject);

	/**
	 * Creates a new ClassProject object.
	 *
	 * @param projectName	the name of the project.
	 * @return a new ClassProject object, or an existing one.
	 */
	ClassProject createClassProject(String projectName);

	/**
	 * Removes a ClassProject instance.
	 * @param projectname is the name of removed ClassProject
	 * @return The removed ClassProject
	 */
	ClassProject removeClassProject(String projectname);
}
