/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.metainfo.api;

import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.api.Logger;

import java.util.Collection;
import java.util.Set;

/**
 * A ClassProject object groups together sets of mapping structures resulting
 * from various mappings of a class and its persistent fields over various
 * persistent models.
 *
 *  Example: let us consider three different mapping structures.
 *  A Client class is first mapped over a relational model. The resulting
 *  mapping structures are kept in a Project named "Poseidon".
 *
 *  The Client class is once more mapped over a relational model. The resulting
 *  mapping structures are kept in a Project named "Corsica".
 *
 *  The Client class is later on mapped over a data file model. The resulting
 *  mapping structures are kept in the Project named "Poseidon".
 *
 */
public interface ClassProject extends MetaObject, Loggable {
	/**
	 * Returns the project name.
	 * @return	 the project name.
	 */
	public String getProjectName();

	/**
	 * Sets the project name.
	 * @param name   the project name.
	 */
	public void setProjectName(String name);

	/**
	 * Returns a collection of Mapping objects.
	 * @return	 a collection.
	 */
	public Collection getMappings();

	/**
	 * Returns a Mapping object.
	 *
	 * @param  mapperName   a mapper name.
	 * @return	 a collection.
	 */
	public Mapping getMapping(String mapperName);

	/**
	 * Returns a set of mapper names.
	 * @return	 a set of mapper names.
	 */
	public Set getMappers();

	/**
	 * Creates a new Mapping object using a MappingFactory object.
	 *
	 * @param mapperName	the name of the mapper.
	 * @return a new Mapping object or an existing one.
	 */
	public Mapping createMapping(String mapperName);

	///////////////////////////////////////////////////////////////////
	//                      from Loggable interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Defines a logger object.
	 * @param logger	the logger object
	 */
	public void setLogger(Logger logger);

	/**
	 * Defines the logger factory to obtain new logger.
	 * @param loggerFactory	the LoggerFactory object to obtain a logger object
	 */
	public void setLoggerFactory(LoggerFactory loggerFactory);

	public Logger getLogger();

	public LoggerFactory getLoggerFactory();
}
