/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package org.objectweb.jorm.metainfo.lib;

import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.Mapping;
import org.objectweb.jorm.metainfo.api.ClassProject;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

import java.util.Collection;
import java.util.HashMap;
import java.util.Set;


/**
 * Implementation of ClassProject.
 *
 * The notion of project allows the developer to define multiple mappings
 * for a class. There is at most one mapping per mapper name in a project.
 */
public class BasicClassProject extends BasicMetaObject
		implements ClassProject, Loggable {
	/**
	 * The project name
	 */
	String projectName;

	/**
	 * This hashmap associates a mapper name with a Mapping object.
	 */
	HashMap mappings;

	/**
	 * a simple logger to log
	 */
	private transient Logger logger;

	/**
	 * Builds a new BasicClassProject object.
	 *
	 * The parent object is a Class object.
	 *
	 * @param projectName   the project name,
	 *        parent	    the parent of the current object.
	 */
	public BasicClassProject(String projectName, MetaObject parent) {
		super(parent);
		this.projectName = projectName;
		mappings = new HashMap();
	}

	///////////////////////////////////////////////////////////////////
	//                      from ClassProject interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Returns the project name.
	 * @return	 the project name.
	 */
	public String getProjectName() {
		return projectName;
	}

	/**
	 * Sets the project name.
	 * @param name   the project name.
	 */
	public void setProjectName(String name) {
		this.projectName = name;
	}

	/**
	 * Returns a collection of Mapping objects.
	 * @return	 a collection.
	 */
	public Collection getMappings() {
		return mappings.values();
	}

	/**
	 * Returns a Mapping object.
	 *
	 * @param  mapperName   a mapper name.
	 * @return	 a collection.
	 */
	public Mapping getMapping(String mapperName) {
		Mapping mapping = (Mapping) mappings.get(mapperName);
		return mapping;
	}

	/**
	 * Returns a set of mapper names.
	 * @return	 a set of mapper names.
	 */
	public Set getMappers() {
		return mappings.keySet();
	}

	/**
	 * Creates a new Mapping object using a MappingFactory object.
	 *
	 * @param mapperName	the name of the mapper.
	 * @return a new Mapping object or an existing one.
	 */
	public Mapping createMapping(String mapperName) {
		if (getLogger().isLoggable(BasicLevel.DEBUG)) {
			getLogger().log(BasicLevel.DEBUG,
							"Create a new Mapping (" + mapperName +
							") for the current ClassProject (" + projectName + ")");
		}
		// verifies whether the Mapping object already exists
		Mapping mp = (Mapping) mappings.get(mapperName);
		if (mp == null) {
			mp = getManager().getMappingFactory(mapperName)
					.createMapping(mapperName, this);
			mp.setLogger(logger);
			mappings.put(mapperName, mp);
		} else {
			String className = ((Class) getParent()).getName();
			getLogger().log(BasicLevel.DEBUG,
							"try to map twice the class (" + className +
							") using the mapper (" + mapperName +
							"), the existing mapping is returned.");
		}
		return mp;
	}

	///////////////////////////////////////////////////////////////////
	//                      from Loggable interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Defines a logger object.
	 * @param logger	the logger object
	 */
	public void setLogger(Logger logger) {
		this.logger = logger;
	}

	/**
	 * Defines the logger factory to obtain new logger.
	 * @param loggerFactory	the LoggerFactory object to obtain a logger object
	 */
	public void setLoggerFactory(LoggerFactory loggerFactory) {
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return null;
	}
}
