/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.metainfo.lib;

import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.CompositeName;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.api.MappingFactory;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.Package;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.type.lib.PTypeSpacePAAH;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * Implementation object of the Manager interface.
 * This interface defines methods to create or find objects which construct
 * the meta information from the description files.
 * This object is managed by the parser system.
 * @author	X. Spengler
 */
public class JormManager implements Manager, Loggable {
	/**
	 * this structure defines all declared schema in the current manager.
	 * key: the name of the schema
	 * value: a Package object
	 */
	protected Map packages;

	/**
	 * this structure defines all the mapping factories declared in the meta
	 * information manager.
	 * key: the name of the mapper
	 * value: a MappingFactory object
	 */
	protected Map mappingFactories;

	/**
	 * a simple logger to log
	 */
	protected transient Logger logger;

	/**
	 * a logger factory to create other loggers if needed
	 */
	protected transient LoggerFactory loggerFactory;

	/**
	 * TBD
	 */
	protected PTypeSpace ptypeSpace;

	public JormManager() {
		init();
	}

	///////////////////////////////////////////////////////////////////
	//                      from Manager interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Initializes the current Manager.
	 * This method replaces the constructor method.
	 */
	public void init() {
		packages = new HashMap();
		mappingFactories = new HashMap();
		ptypeSpace = new PTypeSpacePAAH();
	}

	/**
	 * Returns an existing schema.
	 * If no Package corresponds to the given name, null is returned.
	 *
	 * @param schemaName	the name of the schema
	 * @return	a Package object. If the schema does not exist, null is returned.
	 */
	public Package getPackage(String schemaName) {
		return (Package) packages.get(schemaName);
	}

	/**
	 * Returns a new schema.
	 * This method is the factory method to create schema object. If the
	 * schemaName schema already exists, it is returned.
	 *
	 * @param schemaName	the name of the schema
	 * @return	a new schema object, or the existing one if already defined
	 */
	public Package createPackage(String schemaName) {
		if (logger.isLoggable(BasicLevel.DEBUG))
			logger.log(BasicLevel.DEBUG,
					   "create a new Package (" + schemaName + ") for the current Manager");

		// verify if the schema is not yet in the database
		if (packages.get(schemaName) != null) {
			if (logger.isLoggable(BasicLevel.DEBUG))
				logger.log(BasicLevel.DEBUG, "The Package already exist ("
											 + schemaName + "), return existing one");
			return (Package) packages.get(schemaName);
		}

		// else create a new one
		Package newPackage = new BasicPackage(schemaName, this);
		((Loggable) newPackage).setLogger(logger);
		packages.put(schemaName, newPackage);
		return newPackage;
	}

	/**
	 * Allows to know all the declared schemas into the current metainformation
	 * manager.
	 * This iterator contains Package object.
	 *
	 * @return	an iterator on registered Package object. If there is no schema,
	 *		    an empty iterator is returned.
	 */
	public Collection getPackages() {
		return packages.values();
	}

	/**
	 * Adds a mapping factory to the current metainformation manager.
	 * A mapping factory is a factory to map object (class, field, ...).
	 * @param mapperName	the name of the mapper (i.e: OR for rdb)
	 * @param factory		the mapping factory to add to the list of
	 *                          existing mapping factories
	 */
	public void addMappingFactory(String mapperName, MappingFactory factory) {
		//factory.init();
		if (factory instanceof Loggable)
			((Loggable) factory).setLogger(logger);
		mappingFactories.put(mapperName, factory);
	}

	/**
	 * Returns a mapping factory corresponding to a mapper name.
	 * If no MappingFactory corresponds to the mapper name, null is returned.
	 *
	 * @param mapperName	the name of the mapper to obtain
	 * @return	a mapping factory. If the mapping factory does not exist for
	 *          the mapper, null is returned.
	 */
	public MappingFactory getMappingFactory(String mapperName) {
		if (logger.isLoggable(BasicLevel.DEBUG))
			logger.log(BasicLevel.DEBUG, "get the mappingFactory from " +
										 mapperName);
		return (MappingFactory) mappingFactories.get(mapperName);
	}

	public Collection getMappingFactories() {
		return mappingFactories.values();
	}

	/**
	 * TBD
	 */
	public PTypeSpace getPTypeSpace() {
		return ptypeSpace;
	}

	/**
	 * build an iterator to iterates all classes of the meta-info
	 * @return a new iterator
	 */
	public Collection getClasses() {
		ArrayList v = new ArrayList();
		Iterator allPackages = packages.values().iterator();
		while (allPackages.hasNext()) {
			v.addAll(((Package) allPackages.next()).getClasses());
		}
		return v;
	}

	/**
	 * build an iterator to iterates all classes of the meta-info
	 * @return a new iterator
	 */
	public Collection getCompositeNames() {
		ArrayList v = new ArrayList();
		Iterator allPackages = packages.values().iterator();
		while (allPackages.hasNext()) {
			v.addAll(((Package) allPackages.next()).getCompositeNames());
		}
		return v;
	}

	public Collection getJormObjects() {
		ArrayList v = new ArrayList();
		Iterator allPackages = packages.values().iterator();
		while (allPackages.hasNext()) {
			Package p = (Package) allPackages.next();
			v.addAll(p.getClasses());
			v.addAll(p.getCompositeNames());
		}
		return v;
	}

	/**
	 * searches a class into all the declared schemas and returns it.
	 * If no class corresponds to this name, null is returned.
	 * @param className	the name of the class to search
	 * @return	the Class object found, null if not found.
	 */
	public Class getClass(String className) {
		if (packages == null || className == null) {
			return null;
		}
		int idx = className.lastIndexOf(".");
		Package p = null;
		if (idx == -1) {
			p = (Package) packages.get("");
			if (p != null) {
				return p.getClass(className);
			}
		} else {
			p = (Package) packages.get(className.substring(0, idx));
			if (p != null) {
				return p.getClass(className.substring(idx + 1));
			}
		}
		return null;
	}

	public Class createClass(String fqclassName) {
		if (packages == null || fqclassName == null) {
			return null;
		}
		int idx = fqclassName.lastIndexOf(".");
		if (idx == -1) {
			return createPackage("").createClass(fqclassName);
		} else {
			return createPackage(fqclassName.substring(0, idx))
					.createClass(fqclassName.substring(idx + 1));
		}
	}

	public CompositeName getCompositeName(String fqcompositeNameName) {
		if (packages == null || fqcompositeNameName == null) {
			return null;
		}
		int idx = fqcompositeNameName.lastIndexOf(".");
		Package p = null;
		if (idx == -1) {
			p = (Package) packages.get("");
			if (p != null) {
				return p.getCompositeName(fqcompositeNameName);
			}
		} else {
			p = (Package) packages.get(fqcompositeNameName.substring(0, idx));
			if (p != null) {
				return p.getCompositeName(fqcompositeNameName.substring(idx + 1));
			}
		}
		return null;
	}

	public CompositeName createCompositeName(String fqcompositeNameName) {
		if (packages == null || fqcompositeNameName == null) {
			return null;
		}
		int idx = fqcompositeNameName.lastIndexOf(".");
		if (idx == -1) {
			return createPackage("").createCompositeName(fqcompositeNameName);
		} else {
			return createPackage(fqcompositeNameName.substring(0, idx))
					.createCompositeName(fqcompositeNameName.substring(idx + 1));
		}
	}
	///////////////////////////////////////////////////////////////////
	//                      from MetaObject interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Allows to know the parent MetaObject of the current MetaObject.
	 *
	 * @return	the MetaObject corresponding to the parent of the
	 *		    current object. If there is no parent, null is returned.
	 */
	public MetaObject getParent() {
		// the manager has not parent, orphan, God ?
		return null;
	}

	/**
	 * Set the parent of the current meta object if it is not yet done
	 * by the constructor of the meta object
	 * @param itsParent	the parent MetaObject of the current object
	 */
	public void setParent(MetaObject itsParent) {
	}


	///////////////////////////////////////////////////////////////////
	//                      from Loggable interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Defines a logger object.
	 *
	 * @param logger	the logger object
	 */
	public void setLogger(Logger logger) {
		this.logger = logger;
		if (packages != null)
			for (Iterator it = packages.values().iterator(); it.hasNext();)
				((Loggable) it.next()).setLogger(logger);
	}

	/**
	 * Defines the logger factory to obtain new logger.
	 *
	 * @param loggerFactory	the LoggerFactory object to obtain a logger object
	 */
	public void setLoggerFactory(LoggerFactory loggerFactory) {
		this.loggerFactory = loggerFactory;
		if (logger == null && loggerFactory != null) {
			setLogger(loggerFactory.getLogger("org.objectweb.jorm.metainfo"));
		}
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}
}
