/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mi2xml.lib;

import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.jorm.mi2xml.api.MappingDomtreeBuilder;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.ClassMapping;
import org.objectweb.jorm.metainfo.api.GenClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import java.util.Map;
import java.util.Hashtable;

/**
 * BasicMappingDomtreeBuilder implements the MappingDomtreeBuilder interface.
 */
public abstract class BasicMappingDomtreeBuilder implements MappingDomtreeBuilder, Loggable {

    /**
     * The current Class object.
     */
    protected Class currentClass;

    /**
     * This HashTable associates a MetaObject with an id value.
     */
    protected Map metaobject2idvalue = new Hashtable();

    /**
     * a simple logger to log
     */
    protected Logger logger;

    /**
     * a logger factory to create other loggers if needed
     */
    protected LoggerFactory loggerFactory;

    ///////////////////////////////////////////////////////////////////
    //                      from MappingDomtreeBuilder interface
    ///////////////////////////////////////////////////////////////////

    /**
     * Assigns the current Class object to the MappingDomtreeBuilder object.
     *
     * @param currentClass	the Jorm meta-object associated to the parsed class.
     */
    public void setCurrentClass(Class currentClass) {
        this.currentClass = currentClass;
    }

    /**
     * Assigns an HashTable object to the MappingDomtreeBuilder object.
     *
     * @param metaobject2idvalue	an HashTable that associates a MetaObject with an id value.
     */
    public void setmetaobject2idvalue(Map metaobject2idvalue) {
        this.metaobject2idvalue = metaobject2idvalue;
    }

    /**
     * Adds a class mapping element to a mapping element.
     * @param   document        an org.w3c.dom.Document object,
     *          mappingElement  a mapping element,
     *          classMapping    a <tt>ClassMapping</tt> object.
     */
    public abstract void processClassMapping(Document document, Element mappingElement,
                                             ClassMapping classMapping) throws PException;

    /**
     * Adds a generic class mapping element to a mapping element.
     * @param   document            an org.w3c.dom.Document object,
     *          mappingElement      a mapping element,
     *          genClassMapping     a <tt>GenClassMapping</tt> object.
     */
    public abstract void processGenClassMapping(Document document, Element mappingElement,
                                                GenClassMapping genClassMapping);

    ///////////////////////////////////////////////////////////////////
    //                      from Loggable interface
    ///////////////////////////////////////////////////////////////////

    /**
     * Defines a logger object.
     *
     * @param logger	the logger object
     */
    public void setLogger(Logger logger) {
        this.logger = logger;
    }

    /**
     * Defines the logger factory to obtain new loggers.
     *
     * @param loggerfactory	The LoggerFactory object to obtain a logger object
     */
    public void setLoggerFactory(LoggerFactory loggerfactory) {
        this.loggerFactory = loggerfactory;
        if (logger == null && loggerFactory != null) {
            logger = loggerFactory.getLogger("org.objectweb.jorm.mi2xml.rdb");
        }
    }

    public Logger getLogger() {
        return logger;
    }

    public LoggerFactory getLoggerFactory() {
        return loggerFactory;
    }
}
