/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.naming.api;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.api.PException;

import java.util.Date;
import java.math.BigDecimal;
import java.math.BigInteger;

/**
 * The <b>PNameCoder</b> defines the external structure of the name it manages,
 * which corresponds to the type involved in the encoding/decoding process.
 * Furthermore, several name format may be supported by a particular name
 * coder.
 * @author	R. Basset, P. D?chamboux
 */
public interface PNameCoder {
	/**
	 * The coding type of the naming context is a <code>byte</code>.
	 */
	static final int CTBYTE = 0x0001;
	/**
	 * The coding type of the naming context is an <code>char</code>.
	 */
	static final int CTCHAR = 0x0002;
	/**
	 * The coding type of the naming context is a <code>short</code>.
	 */
	static final int CTSHORT = 0x0004;
	/**
	 * The coding type of the naming context is an <code>int</code>.
	 */
	static final int CTINT = 0x0008;
	/**
	 * The coding type of the naming context is a <code>long</code>.
	 */
	static final int CTLONG = 0x0010;
	/**
	 * The coding type of the naming context is a <code>byte</code>.
	 */
	static final int CTOBYTE = 0x0020;
	/**
	 * The coding type of the naming context is an <code>char</code>.
	 */
	static final int CTOCHAR = 0x0040;
	/**
	 * The coding type of the naming context is a <code>short</code>.
	 */
	static final int CTOSHORT = 0x0080;
	/**
	 * The coding type of the naming context is an <code>int</code>.
	 */
	static final int CTOINT = 0x0100;
	/**
	 * The coding type of the naming context is a <code>long</code>.
	 */
	static final int CTOLONG = 0x0200;
	/**
	 * The coding type of the naming context is a <code>string</code>.
	 */
	static final int CTSTRING = 0x0400;
	/**
	 * The coding type of the naming context is a <code>date</code>.
	 */
	static final int CTDATE = 0x0800;
	/**
	 * The coding type of the naming context is a <code>chararray</code>.
	 */
	static final int CTCHARARRAY = 0x1000;
	/**
	 * The coding type of the naming context is a <code>bytearray</code>.
	 */
	static final int CTBYTEARRAY = 0x2000;
	/**
	 * The coding type of the naming context is a <code>bytearray</code>.
	 */
	static final int CTBIGINTEGER = 0x4000;
	/**
	 * The coding type of the naming context is a <code>bytearray</code>.
	 */
	static final int CTBIGDECIMAL = 0x8000;
	/**
	 * The coding type of the naming context is an <code>Object</code> that
	 * is DS-specific.
	 */
	static final int CTSYSTEM = 0x10000;
	/**
	 * The coding type of the naming context is an <code>Object</code> that
	 * is composite name (e.g., a multi-fields relational primary key).
	 */
	static final int CTCOMPOSITE = 0x80000;

	/**
	 * It tests if a given coding type is supported by this naming context.
	 * @param codingtype	One of the coding type defined by CTxxx constants
	 * 						defined within this interface.
	 * @return	It returns true if the coding type is supported by this naming
	 * 			context.
	 */
	boolean codingSupported(int codingtype);

	/**
	 * The <b>decode</b> methods recreates PName within a naming
	 * context from an encoded representation. In the case of this method, the
	 * encoded representation is an array of bytes. It must always be supported
	 * by a naming context.
	 * @param en	The byte to be decoded.
	 * @return	The resulting PName.
	 */
	PName decode(byte[] en) throws PExceptionNaming;

	/**
	 * The <b>decodeAbstract</b> methods recreates PName within a naming
	 * context from an encoded representation. In the case of this method, the
	 * encoded representation is abstract, which means that this is a Java
	 * object of any complexity. One example of such object is a getter
	 * interface representing a relational primary key (also called composite
	 * names).
	 * @param context A context used to decode the related data.
	 * @param en	The Object to be decoded.
	 * @return	The resulting PName.
	 */
	PName decodeAbstract(Object en, Object context) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>decodeByte</b> methods recreates PName within a naming
	 * context from an encoded representation. In the case of this method, the
	 * encoded representation is a byte.
	 * @param en	The byte to be decoded.
	 * @return	The resulting PName.
	 */
	PName decodeByte(byte en) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>decodeObyte</b> methods recreates PName within a naming
	 * context from an encoded representation. In the case of this method, the
	 * encoded representation is a byte.
	 * @param en	The Byte to be decoded.
	 * @return	The resulting PName.
	 */
	PName decodeObyte(Byte en) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>decodeChar</b> methods recreates PName within a naming
	 * context from an encoded representation. In the case of this method, the
	 * encoded representation is a char.
	 * @param en	The char to be decoded.
	 * @return	The resulting PName.
	 */
	PName decodeChar(char en) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>decodeOchar</b> methods recreates PName within a naming
	 * context from an encoded representation. In the case of this method, the
	 * encoded representation is a Character.
	 * @param en	The Character to be decoded.
	 * @return	The resulting PName.
	 */
	PName decodeOchar(Character en) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>decodeInt</b> methods recreates PName within a naming
	 * context from an encoded representation. In the case of this method, the
	 * encoded representation is an int.
	 * @param en	The int to be decoded.
	 * @return	The resulting PName.
	 */
	PName decodeInt(int en) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>decodeOint</b> methods recreates PName within a naming
	 * context from an encoded representation. In the case of this method, the
	 * encoded representation is an Integer.
	 * @param en	The Integer to be decoded.
	 * @return	The resulting PName.
	 */
	PName decodeOint(Integer en) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>decodeLong</b> methods recreates PName within a naming
	 * context from an encoded representation. In the case of this method, the
	 * encoded representation is a long.
	 * @param en	The long to be decoded.
	 * @return	The resulting PName.
	 */
	PName decodeLong(long en) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>decodeOlong</b> methods recreates PName within a naming
	 * context from an encoded representation. In the case of this method, the
	 * encoded representation is a Long.
	 * @param en	The Long to be decoded.
	 * @return	The resulting PName.
	 */
	PName decodeOlong(Long en) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>decodeShort</b> methods recreates PName within a naming
	 * context from an encoded representation. In the case of this method, the
	 * encoded representation is a short.
	 * @param en	The short to be decoded.
	 * @return	The resulting PName.
	 */
	PName decodeShort(short en) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>decodeOshort</b> methods recreates PName within a naming
	 * context from an encoded representation. In the case of this method, the
	 * encoded representation is a Short.
	 * @param en	The Short to be decoded.
	 * @return	The resulting PName.
	 */
	PName decodeOshort(Short en) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>decodeString</b> methods recreates PName within a naming
	 * context from an encoded representation. In the case of this method, the
	 * encoded representation is a String.
	 * @param en	The String to be decoded.
	 * @return	The resulting PName.
	 */
	PName decodeString(String en) throws PExceptionNaming;

	/**
	 * The <b>decodeCharArray</b> methods recreates PName within a naming
	 * context from an encoded representation. In the case of this method, the
	 * encoded representation is a String.
	 * @param en	The char[] to be decoded.
	 * @return	The resulting PName.
	 */
	PName decodeCharArray(char[] en) throws PExceptionNaming;

	/**
	 * The <b>decodeDate</b> methods recreates PName within a naming
	 * context from an encoded representation. In the case of this method, the
	 * encoded representation is a Date.
	 * @param en	The Date to be decoded.
	 * @return	The resulting PName.
	 */
	PName decodeDate(Date en) throws PExceptionNaming;

	/**
	 * The <b>decodeBigInteger</b> methods recreates PName within a naming
	 * context from an encoded representation. In the case of this method, the
	 * encoded representation is a BigInteger.
	 * @param en	The BigInteger to be decoded.
	 * @return	The resulting PName.
	 */
	PName decodeBigInteger(BigInteger en) throws PExceptionNaming;

	/**
	 * The <b>decodeBigDecimal</b> methods recreates PName within a naming
	 * context from an encoded representation. In the case of this method, the
	 * encoded representation is a BigDecimal.
	 * @param en	The BigDecimal to be decoded.
	 * @return	The resulting PName.
	 */
	PName decodeBigDecimal(BigDecimal en) throws PExceptionNaming;

	/**
	 * The <b>encode</b> method produces an encoded representation of a
	 * PName within a naming context. The encoding format in the case of this
	 * method is an array of bytes. It must be always supported by a naming
	 * context.
	 * @param pn	The PName to be encoded
	 * @return	The array of bytes corresponding to the encoded representation.
	 */
	byte[] encode(PName pn) throws PExceptionNaming;

	/**
	 * The <b>encodeAbstract</b> method produces an encoded representation of a PName
	 * within a naming context. The encoding format is the case of this method
	 * is a Java object of any complexity. Example of such object is a getter
	 * interface giving access of the fields of a relational primary key (also
	 * called composite names).
	 * @param pn	The PName to be encoded.
	 * @return	The Object corresponding to the encoded representation.
	 */
	Object encodeAbstract(PName pn) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeByte</b> method produces an encoded representation of a
	 * PName within a naming context. The encoding format in the case of this
	 * method is a byte.
	 * @param pn	The PName to be encoded
	 * @return	The byte corresponding to the encoded representation.
	 */
	byte encodeByte(PName pn) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeObyte</b> method produces an encoded representation of a
	 * PName within a naming context. The encoding format in the case of this
	 * method is a Byte.
	 * @param pn	The PName to be encoded
	 * @return	The Byte corresponding to the encoded representation.
	 */
	Byte encodeObyte(PName pn) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeChar</b> method produces an encoded representation of a
	 * PName within a naming context. The encoding format in the case of this
	 * method is a char.
	 * @param pn	The PName to be encoded
	 * @return	The char corresponding to the encoded representation.
	 */
	char encodeChar(PName pn) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeOchar</b> method produces an encoded representation of a
	 * PName within a naming context. The encoding format in the case of this
	 * method is a Character.
	 * @param pn	The PName to be encoded
	 * @return	The Character corresponding to the encoded representation.
	 */
	Character encodeOchar(PName pn) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeInt</b> method produces an encoded representation of a
	 * PName within a naming context. The encoding format in the case of this
	 * method is a int.
	 * @param pn	The PName to be encoded
	 * @return	The int corresponding to the encoded representation.
	 */
	int encodeInt(PName pn) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeOint</b> method produces an encoded representation of a
	 * PName within a naming context. The encoding format in the case of this
	 * method is a Integer.
	 * @param pn	The PName to be encoded
	 * @return	The Integer corresponding to the encoded representation.
	 */
	Integer encodeOint(PName pn) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeLong</b> method produces an encoded representation of a
	 * PName within a naming context. The encoding format in the case of this
	 * method is a long.
	 * @param pn	The PName to be encoded
	 * @return	The long corresponding to the encoded representation.
	 */
	long encodeLong(PName pn) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeOlong</b> method produces an encoded representation of a
	 * PName within a naming context. The encoding format in the case of this
	 * method is a Long.
	 * @param pn	The PName to be encoded
	 * @return	The Long corresponding to the encoded representation.
	 */
	Long encodeOlong(PName pn) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeShort</b> method produces an encoded representation of a
	 * PName within a naming context. The encoding format in the case of this
	 * method is a short.
	 * @param pn	The PName to be encoded
	 * @return	The short corresponding to the encoded representation.
	 */
	short encodeShort(PName pn) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeOshort</b> method produces an encoded representation of a
	 * PName within a naming context. The encoding format in the case of this
	 * method is a Short.
	 * @param pn	The PName to be encoded
	 * @return	The Short corresponding to the encoded representation.
	 */
	Short encodeOshort(PName pn) throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeString</b> method produces an encoded representation of a
	 * PName within a naming context. The encoding format in the case of this
	 * method is a String.
	 * @param pn	The PName to be encoded
	 * @return	The String corresponding to the encoded representation.
	 */
	String encodeString(PName pn) throws PExceptionNaming;

	/**
	 * The <b>encodeCharArray</b> method produces an encoded representation of a
	 * PName within a naming context. The encoding format in the case of this
	 * method is a char[].
	 * @param pn	The PName to be encoded
	 * @return	The char[] corresponding to the encoded representation.
	 */
	char[] encodeCharArray(PName pn) throws PExceptionNaming;

	/**
	 * The <b>encodeDate</b> method produces an encoded representation of a
	 * PName within a naming context. The encoding format in the case of this
	 * method is a Date.
	 * @param pn	The PName to be encoded
	 * @return	The Date corresponding to the encoded representation.
	 */
	Date encodeDate(PName pn) throws PExceptionNaming;

	/**
	 * The <b>encodeBigInteger</b> method produces an encoded representation of a
	 * PName within a naming context. The encoding format in the case of this
	 * method is a BigInteger.
	 * @param pn	The PName to be encoded
	 * @return	The BigInteger corresponding to the encoded representation.
	 */
	BigInteger encodeBigInteger(PName pn) throws PExceptionNaming;

	/**
	 * The <b>encodeBigDecimal</b> method produces an encoded representation of a
	 * PName within a naming context. The encoding format in the case of this
	 * method is a BigDecimal.
	 * @param pn	The PName to be encoded
	 * @return	The BigDecimal corresponding to the encoded representation.
	 */
	BigDecimal encodeBigDecimal(PName pn) throws PExceptionNaming;

	/**
	 * It yields a "null" representation of a name within this naming context.
	 * @return	The "null" PName associated with this naming context.
	 */
	PName getNull();

	/**
	 * It assignes a object which defines a null representation of a name
	 * within this naming context.
	 * @param o can be a PNameGetter (generic or generated), a pname, or
	 * a simple value (ex: Integer).
	 */
	void setNullPName(Object o) throws PException;

	/**
	 * Tests if this naming context supports comosite name through a dynamic
	 * approach, which means that encodingAbstract/decodingAbstract use getter
	 * objects that implement the PNameGetter interface.
	 * @return	true if dynamic getter is supported by composite name coding.
	 */
	boolean supportDynamicComposite();

	/**
	 * In case of a naming context that supports composite names (i.e., names
	 * composed of several typed fields, whose type is limited to scalar ones),
	 * it tests if a particular field is defined by this composite name.
	 * It should be used by PClassMapping objects in order to verify, at
	 * deployment time, that naming contexts that are defined to manage the
	 * various references supports the same encoding name structure.
	 * @return	true if the proposed field is supported by this composite name.
	 */
	boolean supportCompositeField(String fn, PType ft);

	/**
	 * Tests if this naming context supports comosite name through a static
	 * approach, which means that encodingAbstract/decodingAbstract use getter
	 * objects that implement specific NsFieldGetter interface generated for
	 * a particular field "Field" within a particular NameDef space "Ns".
	 * @return	true if static getter is supported by composite name coding.
	 */
	boolean supportStaticComposite();

	/**
	 * Retrieves the PType associated to the JORM class which is the type of
	 * the name managed by this naming context.
	 * @return	The related PType.
	 */
	PType getPType();


	/**
	 * Assigns the PType associated to the JORM class which is the type of
	 * the name managed by this naming context.
	 * @param pt	The PType of the JORM class reference managed by this naming
	 * 				context.
	 */
	void setPType(PType pt);
}
