/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.naming.api;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.type.api.PExceptionTyping;

import java.util.Date;
import java.math.BigDecimal;
import java.math.BigInteger;

/**
 * The <b>PNameGetter</b> interface defines a generic accessor to be used by
 * persistent bindings to access fields of composite names it manages.
 * @author	P. D?chamboux
 */
public interface PNameGetter {
	/**
	 * It retrieves the value of a byte field.
	 * @param fn	A String containing the name of the field.
	 * @param ctxt	A context used by PNameGetter to extract the related data.
	 * @return	The byte value of the field as stored within the composite
	 * 			name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	byte pngetByteField(String fn, Object ctxt) throws PException;

	/**
	 * It retrieves the value of a Byte field.
	 * @param fn	A String containing the name of the field.
	 * @param ctxt	A context used by PNameGetter to extract the related data.
	 * @return	The Byte value of the field as stored within the composite
	 * 			name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	Byte pngetObyteField(String fn, Object ctxt) throws PException;

	/**
	 * It retrieves the value of a char field.
	 * @param fn	A String containing the name of the field.
	 * @param ctxt	A context used by PNameGetter to extract the related data.
	 * @return	The char value of the field as stored within the composite
	 * 			name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	char pngetCharField(String fn, Object ctxt) throws PException;

	/**
	 * It retrieves the value of a Character field.
	 * @param fn	A String containing the name of the field.
	 * @param ctxt	A context used by PNameGetter to extract the related data.
	 * @return	The Character value of the field as stored within the composite
	 * 			name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	Character pngetOcharField(String fn, Object ctxt) throws PException;

	/**
	 * It retrieves the value of a short field.
	 * @param fn	A String containing the name of the field.
	 * @param ctxt A context used by PNameGetter to extract the related data.
	 *                In in the case of RDB, it can be the result set.
	 * @return	The short value of the field as stored within the composite
	 * 			name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	short pngetShortField(String fn, Object ctxt) throws PException;

	/**
	 * It retrieves the value of a Short field.
	 * @param fn	A String containing the name of the field.
	 * @param ctxt	A context used by PNameGetter to extract the related data.
	 * @return	The Short value of the field as stored within the composite
	 * 			name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	Short pngetOshortField(String fn, Object ctxt) throws PException;

	/**
	 * It retrieves the value of an int field.
	 * @param fn	A String containing the name of the field.
	 * @param ctxt	A context used by PNameGetter to extract the related data.
	 * @return	The int value of the field as stored within the composite
	 * 			name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	int pngetIntField(String fn, Object ctxt) throws PException;

	/**
	 * It retrieves the value of an Integer field.
	 * @param fn	A String containing the name of the field.
	 * @param ctxt	A context used by PNameGetter to extract the related data.
	 * @return	The Integer value of the field as stored within the composite
	 * 			name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	Integer pngetOintField(String fn, Object ctxt) throws PException;

	/**
	 * It retrieves the value of a long field.
	 * @param ctxt	A context used by PNameGetter to extract the related data.
	 * @param fn	A String containing the name of the field.
	 * @return	The long value of the field as stored within the composite
	 * 			name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	long pngetLongField(String fn, Object ctxt) throws PException;

	/**
	 * It retrieves the value of a Long field.
	 * @param fn	A String containing the name of the field.
	 * @param ctxt	A context used by PNameGetter to extract the related data.
	 * @return	The Long value of the field as stored within the composite
	 * 			name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	Long pngetOlongField(String fn, Object ctxt) throws PException;

	/**
	 * It retrieves the value of a string field.
	 * @param ctxt	A context used by PNameGetter to extract the related data.
	 * @param fn	A String containing the name of the field.
	 * @return	The java.lang.String value of the field as stored within the
	 * 			composite name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	String pngetStringField(String fn, Object ctxt) throws PException;

	/**
	 * It retrieves the value of a byte[] field.
	 * @param ctxt	A context used by PNameGetter to extract the related data.
	 * @param fn	A String containing the name of the field.
	 * @return	The byte[] value of the field as stored within the
	 * 			composite name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	byte[] pngetByteArrayField(String fn, Object ctxt) throws PException;

	/**
	 * It retrieves the value of a char[] field.
	 * @param ctxt	A context used by PNameGetter to extract the related data.
	 * @param fn	A String containing the name of the field.
	 * @return	The char[] value of the field as stored within the
	 * 			composite name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	char[] pngetCharArrayField(String fn, Object ctxt) throws PException;

	/**
	 * It retrieves the value of a date field.
	 * @param fn	A String containing the name of the field.
	 * @param ctxt	A context used by PNameGetter to extract the related data.
	 * @return	The java.util.Date value of the field as stored within the
	 * 			composite name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	Date pngetDateField(String fn, Object ctxt) throws PException;

	/**
	 * It retrieves the value of a BigInteger field.
	 * @param fn	A String containing the name of the field.
	 * @param ctxt	A context used by PNameGetter to extract the related data.
	 * @return	The java.math.BigInteger value of the field as stored within the
	 * 			composite name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	BigInteger pngetBigIntegerField(String fn, Object ctxt) throws PException;

	/**
	 * It retrieves the value of a BigDecimal field.
	 * @param fn	A String containing the name of the field.
	 * @param ctxt	A context used by PNameGetter to extract the related data.
	 * @return	The java.math.BigDecimal value of the field as stored within the
	 * 			composite name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	BigDecimal pngetBigDecimalField(String fn, Object ctxt) throws PException;
}
