/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.type.api;

import java.io.Serializable;

/**
 * The <b>PTypeSpace</b> interface defines a space of JORM type along with
 * the inheritance relationships that may exist between them.
 * @author	P. D?chamboux
 */
public interface PTypeSpace extends Serializable {
	final static PType BOOLEAN = new PType(PType.TYPECODE_BOOLEAN,
										   "boolean", "boolean", "PTypeSpace.BOOLEAN", null);
	final static PType CHAR = new PType(PType.TYPECODE_CHAR,
										"char", "char", "PTypeSpace.CHAR", "Char");
	final static PType BYTE = new PType(PType.TYPECODE_BYTE,
										"byte", "byte", "PTypeSpace.BYTE", "Byte");
	final static PType SHORT = new PType(PType.TYPECODE_SHORT,
										 "short", "short", "PTypeSpace.SHORT", "Short");
	final static PType INT = new PType(PType.TYPECODE_INT,
									   "int", "int", "PTypeSpace.INT", "Int");
	final static PType LONG = new PType(PType.TYPECODE_LONG,
										"long", "long", "PTypeSpace.LONG", "Long");
	final static PType FLOAT = new PType(PType.TYPECODE_FLOAT,
										 "float", "float", "PTypeSpace.FLOAT", null);
	final static PType DOUBLE = new PType(PType.TYPECODE_DOUBLE,
										  "double", "double", "PTypeSpace.DOUBLE", null);
	final static PType OBJBOOLEAN = new PType(PType.TYPECODE_OBJBOOLEAN,
											  "Boolean", "java.lang.Boolean", "PTypeSpace.OBJBOOLEAN", null);
	final static PType OBJCHAR = new PType(PType.TYPECODE_OBJCHAR,
										   "Char", "java.lang.Character", "PTypeSpace.OBJCHAR", "Ochar");
	final static PType OBJBYTE = new PType(PType.TYPECODE_OBJBYTE,
										   "Byte", "java.lang.Byte", "PTypeSpace.OBJBYTE", "Obyte");
	final static PType OBJSHORT = new PType(PType.TYPECODE_OBJSHORT,
											"Short", "java.lang.Short", "PTypeSpace.OBJSHORT", "Oshort");
	final static PType OBJINT = new PType(PType.TYPECODE_OBJINT,
										  "Int", "java.lang.Integer", "PTypeSpace.OBJINT", "Oint");
	final static PType OBJLONG = new PType(PType.TYPECODE_OBJLONG,
										   "Long", "java.lang.Long", "PTypeSpace.OBJLONG", "Olong");
	final static PType OBJFLOAT = new PType(PType.TYPECODE_OBJFLOAT,
											"Float", "java.lang.Float", "PTypeSpace.OBJFLOAT", null);
	final static PType OBJDOUBLE = new PType(PType.TYPECODE_OBJDOUBLE,
											 "Double", "java.lang.Double", "PTypeSpace.OBJDOUBLE", null);
	final static PType STRING = new PType(PType.TYPECODE_STRING,
										  "string", "java.lang.String", "PTypeSpace.STRING", "String");
	final static PType DATE = new PType(PType.TYPECODE_DATE,
										"date", "java.util.Date", "PTypeSpace.DATE", "Date");
	final static PType CHARARRAY = new PType(PType.TYPECODE_CHARARRAY,
											 "chararray", "char[]", "PTypeSpace.CHARARRAY", "CharArray");
	final static PType BYTEARRAY = new PType(PType.TYPECODE_BYTEARRAY,
											 "bytearray", "byte[]", "PTypeSpace.BYTEARRAY", "");
	final static PType SERIALIZED = new PType(PType.TYPECODE_SERIALIZED,
											  "serialized", "java.io.Serializable", "PTypeSpace.SERIALIZED", null);
	final static PType BIGINTEGER = new PType(PType.TYPECODE_BIGINTEGER,
											  "BigInteger", "java.math.BigInteger", "PTypeSpace.BIGINTEGER", "BigInteger");
	final static PType BIGDECIMAL = new PType(PType.TYPECODE_BIGDECIMAL,
											  "BigDecimal", "java.math.BigDecimal", "PTypeSpace.BIGDECIMAL", "BigDecimal");
	final static PType REFTOP = new PType(PType.TYPECODE_REFERENCE,
										  "object", "java.lang.Object", "PTypeSpace.REFTOP", null);
	final static PType[] PREDEFINEDPTYPES = {
		BOOLEAN, BYTE, CHAR, SHORT, INT, LONG, FLOAT, DOUBLE,
		OBJBOOLEAN, OBJBYTE, OBJCHAR, OBJSHORT, OBJINT, OBJLONG, OBJFLOAT, OBJDOUBLE,
		STRING, DATE, SERIALIZED, BIGINTEGER, BIGDECIMAL, REFTOP, CHARARRAY, BYTEARRAY
	};
	final static PType[] SCALARPTYPES = {
		BOOLEAN, BYTE, CHAR, SHORT, INT, LONG,
		OBJBOOLEAN, OBJBYTE, OBJCHAR, OBJSHORT, OBJINT, OBJLONG,
		STRING, DATE, CHARARRAY, BYTEARRAY, BIGINTEGER, BIGDECIMAL
	};

	/**
	 *
	 */
	final static PType[] NULLSUPERTYPES = new PType[0];

	/**
	 *
	 */
	final static String[][] NULLSUPERCLASSNAMES = new String[0][];

	/**
	 * It creates a new PType for a JORM class that has no inherited class.
	 * @param classname		The name of the JORM class.
	 * @return	The created PType or the existing one if it has the same
	 * 			definition (same name and same super types).
	 * @exception PExceptionTyping	Raised when a PType with this class name
	 * 								exists with a different array of super
	 * 								types.
	 */
	PType createPType(String classname) throws PExceptionTyping;

	/**
	 * It creates a new PType for a JORM class. It is defined by its name and
	 * its inheritance hierarchy. For example, if "Engineer" inherits from
	 * "Employee" that inherits from "Person", then the following stands:
	 * <bl>
	 * <li>classname = "schm1.Engineer"</li>
	 * <li>superclassnames = {{"schm1.Engineer", "schm2.Employee"},
	 * {"schm2.Employee", "schm3.Person"}}</li>
	 * </bl>
	 * @param classname			The name of the JORM class to be defined.
	 * @param superclassnames	The hierarchy of of classes inherited by the
	 * 							previous class (defined by classname) if any.
	 * @return	The created PType or the existing one if it has the same
	 * 			definition (same name and same super types).
	 * @exception PExceptionTyping	Raised when a PType with this class name
	 * 								exists with a different array of super
	 * 								types.
	 */
	PType createPType(String classname, String[][] superclassnames)
			throws PExceptionTyping;

	/**
	 * It creates a new PType for a JORM generic class instance. Such an
	 * instance may have a nested structure like
	 * "list<array<set<int>>>", which means that:
	 * <bl>
	 * <li>gcnames = {"list", "array", "set"}</li>
	 * <li>ptypename = INT</li>
	 * </bl>
	 * @param gcnames	The array of names of the nested generic classes.
	 * @param ptype 	The PType of the type of the elements contained
	 * 					within this generic class, which is one of those
	 * 					defined inside PREDEFINEDPTYPES.
	 * @return	The created PType or the existing one if it has the same
	 * 			definition (same name and same element type).
	 * @exception PExceptionTyping	Raised when a PType with this class name
	 * 								exists with a different inner PType
	 */
	PType createPType(String[] gcnames, PType ptype) throws PExceptionTyping;

	/**
	 * It creates a new PType for a JORM generic class instance. Such an
	 * instance may have a nested structure like
	 * "list<array<set<schm1.Engineer>>>", which means that:
	 * <bl>
	 * <li>gcnames = {"list", "array", "set"}</li>
	 * <li>typename = "schm1.Engineer"</li>
	 * <li>superclassnames = {{"schm1.Engineer", "schm2.Employee"},
	 * {"schm2.Employee", "schm3.Person"}}</li>
	 * </bl>
	 * @param gcnames			The array of names of the nested generic
	 * 							classes.
	 * @param typename			The name of the type of the elements contained
	 * 							within this generic class.
	 * @param superclassnames	The hierarchy of of classes inherited by the
	 * 							previous class (defined by typename) if any.
	 * @return	The created PType or the existing one if it has the same
	 * 			definition (same name and same element type).
	 * @exception PExceptionTyping	Raised when a PType with this class name
	 * 								exists with a different inner PType
	 */
	PType createPType(String[] gcnames, String typename,
					  String[][] superclassnames) throws PExceptionTyping;

	/**
	 * It tries to retrieve a PType representing a JORM class with the given
	 * class name.
	 * @param classname		The name of a JORM class the PType is looking for.
	 * @return	The relevant PType if one has been found or null if none found.
	 */
	PType getPType(String classname);

	/**
	 * Retrieves all subtypes of this PType defined within its PTypeSpace.
	 * @param pt	The concerned PType.
	 * @return	The array of PType containing all these subtypes.
	 */
	PType[] getSubTypes(PType pt);

	/**
	 * It compares to PType representing JORM classes. It verifies if the first
	 * one conforms to the second one.
	 * @param pt1	The first JORM class PType.
	 * @param pt2	The second JORM class PType.
	 * @return	It returns true if pt1 conforms to pt2.
	 */
	boolean isa(PType pt1, PType pt2);
}
