/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.util.io.lib;

import org.objectweb.jorm.util.io.api.PathExplorer;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

import java.util.Iterator;
import java.util.Collection;
import java.util.ArrayList;
import java.util.Enumeration;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.FileInputStream;
import java.io.File;
import java.io.IOException;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;

/**
 * DirJavaExplorer defines methods to create object able
 * to manage a list of paths and furnish input and output stream
 * on files
 */
public class DirJavaExplorer implements PathExplorer, Loggable {
	/**
	 * the vector which contains all the paths
	 */
	private Collection paths;
	private Logger logger;
	private LoggerFactory loggerFactory;

	static final String LOGGER_NAME = "org.objectweb.jorm.io.pathexplorer";
	static String fileSeparator = System.getProperty("file.separator");
	static String pathSeparator = System.getProperty("path.separator");

	/**
	 * Builds a new DirJavaExplorer.
	 */
	public DirJavaExplorer() {
		paths = new ArrayList();
	}

	public void setLogger(Logger logger) {
		this.logger = logger;
	}

	/**
	 * Returns a logger to an component that wants to log things.
	 */
	public Logger getLogger() {
		return logger;
	}

	/**
	 * Returns a logger factory that allows the creation of new loggers.
	 */
	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}

	/**
	 * Assigns a logger factory that allows the creation of new loggers.
	 *
	 * @param loggerfactory	the LoggerFactory object to obtain new loggers
	 */
	public void setLoggerFactory(LoggerFactory loggerfactory) {
		this.loggerFactory = loggerFactory;
		if (logger == null && this.loggerFactory != null)
			logger = loggerFactory.getLogger(LOGGER_NAME);
	}

	/**
	 * Adds a classpath and its path separator to the current explorer.
	 * This classpath is added to the list of path already defines.
	 *
	 * @param cpath	    the classpath which contains a list of paths
	 */
	public void addPath(String cpath) {
		int previous, current;
		previous = 0;
		current = cpath.indexOf(pathSeparator, previous);
		while (current != -1) {
			paths.add(cpath.substring(previous, current));

			previous = current + 1;
			current = cpath.indexOf(pathSeparator, previous);
		}
		paths.add(cpath.substring(previous));
	}

	/**
	 * Creates the paths with the given path.
	 *
	 * @param path	the vector of paths (which are String)
	 */
	public void addPath(Collection path) {
		paths.addAll(path);
	}

	/**
	 * Gets an input stream from a given file. This file is opened from the
	 * defined path.
	 *
	 * @param file	the string representation of the name of the file
	 * @return	    an InputStream for the given file
	 */
	public InputStream getInputStream(String file) {
		if (logger != null)
			logger.log(BasicLevel.DEBUG, "Search " + file);
		try {
			File f = null;
			for (Iterator it = paths.iterator(); it.hasNext();) {
				String path = (String) it.next();
				if (logger != null)
					logger.log(BasicLevel.DEBUG, "Try with " + path);
				File p = new File(path);
				if (p.exists()) {
					if (p.isDirectory()) {
						f = new File(p, file);
						if (f.exists()) {
							if (logger != null)
								logger.log(BasicLevel.DEBUG, "Found " + f.getAbsolutePath());
							return new FileInputStream(f);
						}
					} else if (p.isFile()) {
						JarFile jarfile = null;
						try {
							jarfile = new JarFile(path);
							ZipEntry zipentry = jarfile.getEntry(file);
							if (zipentry == null)
								zipentry = jarfile.getEntry(file.replace('\\', '/'));
							if (zipentry == null)
								zipentry = jarfile.getEntry(file.replace('/', '\\'));
							if (zipentry != null) {
								if (logger != null)
									logger.log(BasicLevel.DEBUG, "Found " + file + " in " + path);
								return jarfile.getInputStream(zipentry);
							} else {
								if (logger != null) {
									logger.log(BasicLevel.DEBUG, "Entry not Found " + file + " in jar file " + path);
									for (Enumeration e = jarfile.entries(); e.hasMoreElements();) {
										logger.log(BasicLevel.DEBUG, "entry: " + e.nextElement());
									}
								}
							}
						} catch (IOException e) {
							if (logger != null)
								logger.log(BasicLevel.WARN, "Unmanaged file entry: " + path);
						}
					} else {
						if (logger != null)
							logger.log(BasicLevel.WARN, "Unmanaged path entry: " + path);
					}
				} else {
					System.out.println("WARN: Unreachable path entry: " + path);
				}
			}
			f = new File(file);
			if (f.exists()) {
				if (logger != null)
					logger.log(BasicLevel.DEBUG, "Found " + f.getAbsolutePath());
				return new FileInputStream(f);
			}

			return null;
		} catch (Exception e) {
			if (logger != null)
				logger.log(BasicLevel.ERROR, "", e);
			return null;
		}
	}

	/**
	 * Gets an output stream from a given file. This file is opened from the
	 * defined path.
	 *
	 * @param file	the string representation of the name of the file
	 * @return	an OutputStream for the given file
	 */
	public OutputStream getOutputStream(String file) {
		return null;
	}

	public String getClassPath() {
		String result = "";
		boolean first = true;
		for (Iterator it = paths.iterator(); it.hasNext();) {
			result += (first ? "" : File.pathSeparator) + it.next();
			first = false;
		}
		return result;
	}
}
