/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package org.objectweb.jorm.xml2mi.lib;

import org.apache.xerces.xni.parser.XMLInputSource;
import org.apache.xerces.xni.parser.XMLEntityResolver;
import org.apache.xerces.xni.XMLResourceIdentifier;
import org.apache.xerces.xni.XNIException;
import org.apache.tools.ant.types.DTDLocation;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.jorm.util.api.Loggable;

import java.io.IOException;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.zip.ZipEntry;
import java.util.jar.JarFile;
import java.net.URL;

/**
 * @author S.Chassande-Barrioz
 */
public class DTDResolver implements XMLEntityResolver, Loggable {

	private ArrayList dtdLocations = null;
	private Logger logger = null;

	public DTDResolver(ArrayList dtdLocations) {
		this.dtdLocations = dtdLocations;
	}

	public DTDResolver(ArrayList dtdLocations, Logger logger) {
		this.dtdLocations = dtdLocations;
		this.logger = logger;
	}

	// IMPLEMENTATION OF METHODS FROM THE XMLEntityResolver INTERFACE

	public XMLInputSource resolveEntity(XMLResourceIdentifier identifier) throws XNIException, IOException {
		String publicid = identifier.getPublicId();
		String location = null;
		if (publicid == null) {
			location = identifier.getBaseSystemId();
		} else {
			Iterator it = dtdLocations.iterator();
			logger.log(BasicLevel.DEBUG, "Looking for the publicId: " + publicid);
			while (it.hasNext()) {
				DTDLocation dtdloc = (DTDLocation) it.next();
				String current = dtdloc.getPublicId();
				logger.log(BasicLevel.DEBUG, "Current public id: " + current);
				if (current.equals(publicid)) {
					location = dtdloc.getLocation();
					break;
				}
			}
		}
		if (location == null) {
			logger.log(BasicLevel.WARN, "No dtd location found for the publicId:" + publicid);
			if (identifier.getBaseSystemId() == null)
				throw new IOException("Could not resolve publicid: [" + publicid + "]");
			logger.log(BasicLevel.WARN, "Resolved " + publicid + " to with default location " + identifier.getBaseSystemId());
			return new XMLInputSource(identifier);
		}

		InputStream is = null;

		// try in the file system
		try {
			is = new FileInputStream(location);
			logger.log(BasicLevel.DEBUG, "Resolved " + publicid + " to local file " + location);
		} catch (FileNotFoundException e) {
			logger.log(BasicLevel.DEBUG, location + " is not a local file ");
		}

		if (is == null) {
			URL urldtd = null;
			try {
				urldtd = new URL(location);
				is = urldtd.openStream();
				logger.log(BasicLevel.DEBUG, "Resolved " + publicid + " to url " + urldtd);
			} catch (IOException e) {
				logger.log(BasicLevel.DEBUG, location + " is not an URL ");
			}
		}

		// Try in the classpath
		if (is == null) {
			is = getClass().getClassLoader().getResourceAsStream(location);
			if (is != null) {
				logger.log(BasicLevel.DEBUG, "Resolved " + publicid + " to resource " + location);
			} else {
				logger.log(BasicLevel.DEBUG, location + " is not availlable in the classpath");
			}
		}

		//the location can be a jar file
		if (is == null) {
			int idx = location.indexOf(".jar!");
			if (idx != -1) {
				try {
					JarFile jarfile = new JarFile(location.substring(0, idx));
					// the path is the jar file, can continue
					if (jarfile != null) {
						ZipEntry zipentry = jarfile.getEntry(
								location.substring(idx + 5, location.length()));
						// if the entry (file) exists in the jar file
						// extracts it and return an input stream from it
						if (zipentry != null) {
							is = jarfile.getInputStream(zipentry);
						}
					}
				} catch (Exception e) {
					logger.log(BasicLevel.DEBUG, location + " is not an jar file");
				}
			}
		}

		if (is != null)
			return new XMLInputSource(publicid,
									  identifier.getLiteralSystemId(),
									  identifier.getBaseSystemId(),
									  is,
									  null);
		else
			throw new IOException("Could not resolve publicid: ["
								  + publicid + "] location [" + location + "]");
	}

	// IMPLEMENTATION OF METHODS FROM THE Loggable INTERFACE //
	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return null;
	}

	public void setLogger(Logger logger) {
		this.logger = logger;
	}

	public void setLoggerFactory(LoggerFactory loggerfactory) {
	}
}
